@@ -1,6 +1,8 @@
-#include <libtcod.h>
+#include <SDL.h>
 #include <stdio.h>
 
+#include <libtcod.hpp>
+
 static float version = 1.0f;
 
 int main(int argc, char* argv[]) {
