@@ -1,59 +1,55 @@
-#include <libtcod.h>
-#include <math.h>
-#include <stdarg.h>
-#include <stdio.h>
+#include <SDL.h>
 
+#include <cmath>
+#include <cstdarg>
+#include <cstdio>
+#include <fstream>
+#include <libtcod.hpp>
 #include <libtcod/gui/gui.hpp>
+#include <libtcod/timer.hpp>
+#include <string>
 
 #include "operation.hpp"
 
-TCODHeightMap *hm = NULL, *hmold = NULL;
+TCODHeightMap* hm = NULL;
+static TCODHeightMap* hm_old = NULL;
 TCODNoise* noise = NULL;
-TCODRandom *rnd = NULL, *backupRnd = NULL;
-TCODConsole* guicon = NULL;
-bool greyscale = false;
-bool slope = false;
-bool normal = false;
+TCODRandom* rnd = NULL;
+static TCODRandom* backupRnd = NULL;
+static TCODConsole* guicon = NULL;
+static bool greyscale = false;
+static bool slope = false;
+static bool normal = false;
 bool isNormalized = true;
-bool oldNormalized = true;
-char msg[512] = "";
+static bool oldNormalized = true;
+static std::string msg = "";
 float msgDelay = 0.0f;
-float hillRadius = 0.1f;
-float hillVariation = 0.5f;
+static constexpr float hillRadius = 0.1f;
+static constexpr float hillVariation = 0.5f;
 float addFbmDelta = 0.0f;
 float scaleFbmDelta = 0.0f;
 uint32_t seed = 0xdeadbeef;
 
-float sandHeight = 0.12f;
-float grassHeight = 0.315f;
-float snowHeight = 0.785f;
+static float sandHeight = 0.12f;
+static float grassHeight = 0.315f;
+static float snowHeight = 0.785f;
 
-char landMassTxt[128] = "";
-char minZTxt[128] = "";
-char maxZTxt[128] = "";
-char seedTxt[128] = "";
+static char landMassTxt[128] = "";
+static char minZTxt[128] = "";
+static char maxZTxt[128] = "";
+static char seedTxt[128] = "";
 float mapmin = 0.0f, mapmax = 0.0f;
-float oldmapmin = 0.0f, oldmapmax = 0.0f;
+static float oldmapmin = 0.0f, oldmapmax = 0.0f;
 
 // ui
 ToolBar* params = NULL;
 ToolBar* history = NULL;
 ToolBar* colorMapGui = NULL;
 
-float voronoiCoef[2] = {-1.0f, 1.0f};
+static float voronoiCoef[2] = {-1.0f, 1.0f};
 
-/* light 3x3 smoothing kernel :
-        1  2 1
-        2 20 2
-        1  2 1
-*/
-int smoothKernelSize = 9;
-int smoothKernelDx[9] = {-1, 0, 1, -1, 0, 1, -1, 0, 1};
-int smoothKernelDy[9] = {-1, -1, -1, 0, 0, 0, 1, 1, 1};
-float smoothKernelWeight[9] = {1, 2, 1, 2, 20, 2, 1, 2, 1};
-
-TCODColor mapGradient[256];
-#define MAX_COLOR_KEY 10
+static TCODColor mapGradient[256];
+static constexpr auto MAX_COLOR_KEY = 10;
 
 // TCOD's land color map
 static int keyIndex[MAX_COLOR_KEY] = {
@@ -66,30 +62,36 @@ static int keyIndex[MAX_COLOR_KEY] = {
     (int)(snowHeight * 255) + 10,
     255};
 static TCODColor keyColor[MAX_COLOR_KEY] = {
-    TCODColor(0, 0, 50),       // deep water
-    TCODColor(30, 30, 170),    // water-sand transition
-    TCODColor(114, 150, 71),   // sand
-    TCODColor(80, 120, 10),    // sand-grass transition
-    TCODColor(17, 109, 7),     // grass
-    TCODColor(120, 220, 120),  // grass-snow transisiton
+    TCODColor(0, 0, 50),  // deep water
+    TCODColor(30, 30, 170),  // water-sand transition
+    TCODColor(114, 150, 71),  // sand
+    TCODColor(80, 120, 10),  // sand-grass transition
+    TCODColor(17, 109, 7),  // grass
+    TCODColor(120, 220, 120),  // grass-snow transistion
     TCODColor(208, 208, 239),  // snow
     TCODColor(255, 255, 255)};
 static Image* keyImages[MAX_COLOR_KEY];
 
-static int nbColorKeys = 8;
+static constexpr int nbColorKeys = 8;
+
+static constexpr auto BLACK = TCODColor{0, 0, 0};
+static constexpr auto WHITE = TCODColor{255, 255, 255};
+static constexpr auto LIGHT_BLUE = TCODColor{63, 63, 255};
+
+static constexpr auto BLACK_ = TCOD_ColorRGB{0, 0, 0};
+static constexpr auto WHITE_ = TCOD_ColorRGB{255, 255, 255};
+static constexpr auto LIGHT_BLUE_ = TCOD_ColorRGB{63, 63, 255};
 
 void initColors() { TCODColor::genMap(mapGradient, nbColorKeys, keyColor, keyIndex); }
 
-void render() {
-  static TCODHeightMap backup(HM_WIDTH, HM_HEIGHT);
+void render(TCOD_Console& console, float delta_time) {
+  TCODHeightMap backup(HM_WIDTH, HM_HEIGHT);
   isNormalized = true;
-  TCODConsole::root->setDefaultBackground(TCODColor::black);
-  TCODConsole::root->setDefaultForeground(TCODColor::white);
-  TCODConsole::root->clear();
+  for (auto& tile : console) tile = {' ', {255, 255, 255, 255}, {0, 0, 0, 255}};
   backup.copy(hm);
   mapmin = 1E8f;
   mapmax = -1E8f;
-  for (int i = 0; i < HM_WIDTH * HM_HEIGHT; i++) {
+  for (int i = 0; i < HM_WIDTH * HM_HEIGHT; ++i) {
     if (hm->values[i] < 0.0f || hm->values[i] > 1.0f) {
       isNormalized = false;
     }
@@ -98,20 +100,18 @@ void render() {
   }
   if (!isNormalized) backup.normalize();
   // render the TCODHeightMap
-  for (int x = 0; x < HM_WIDTH; x++) {
-    for (int y = 0; y < HM_HEIGHT; y++) {
+  for (int y = 0; y < HM_HEIGHT; ++y) {
+    for (int x = 0; x < HM_WIDTH; ++x) {
       float z = backup.getValue(x, y);
       uint8_t val = (uint8_t)(z * 255);
       if (slope) {
         // render the slope map
         z = CLAMP(0.0f, 1.0f, hm->getSlope(x, y) * 10.0f);
         val = (uint8_t)(z * 255);
-        TCODColor c(val, val, val);
-        TCODConsole::root->setCharBackground(x, y, c);
+        console.at({x, y}).bg = {val, val, val, 255};
       } else if (greyscale) {
         // render the greyscale heightmap
-        TCODColor c(val, val, val);
-        TCODConsole::root->setCharBackground(x, y, c);
+        console.at({x, y}).bg = {val, val, val, 255};
       } else if (normal) {
         // render the normal map
         float n[3];
@@ -119,45 +119,42 @@ void render() {
         uint8_t r = (uint8_t)((n[0] * 0.5f + 0.5f) * 255);
         uint8_t g = (uint8_t)((n[1] * 0.5f + 0.5f) * 255);
         uint8_t b = (uint8_t)((n[2] * 0.5f + 0.5f) * 255);
-        TCODColor c(r, g, b);
-        TCODConsole::root->setCharBackground(x, y, c);
+        console.at({x, y}).bg = {r, g, b, 255};
       } else {
         // render the colored heightmap
-        TCODConsole::root->setCharBackground(x, y, mapGradient[val]);
+        console.at({x, y}).bg = {mapGradient[val].r, mapGradient[val].g, mapGradient[val].b, 255};
       }
     }
   }
-  sprintf(minZTxt, "min z    : %.2f", mapmin);
-  sprintf(maxZTxt, "max z    : %.2f", mapmax);
-  sprintf(seedTxt, "seed     : %X", seed);
-  float landProportion = 100.0f - 100.0f * backup.countCells(0.0f, sandHeight) / (hm->w * hm->h);
-  sprintf(landMassTxt, "landMass : %d %%", (int)landProportion);
-  if (!isNormalized)
-    TCODConsole::root->printf(HM_WIDTH / 2, HM_HEIGHT - 1, TCOD_BKGND_NONE, TCOD_CENTER, "the map is not normalized !");
+  snprintf(minZTxt, sizeof(minZTxt), "min z    : %.2f", mapmin);
+  snprintf(maxZTxt, sizeof(maxZTxt), "max z    : %.2f", mapmax);
+  snprintf(seedTxt, sizeof(seedTxt), "seed     : %X", seed);
+  const float landProportion = 100.0f - 100.0f * backup.countCells(0.0f, sandHeight) / (hm->w * hm->h);
+  snprintf(landMassTxt, sizeof(landMassTxt), "landMass : %d %%", (int)landProportion);
+  if (!isNormalized) {
+    tcod::print(
+        console, {HM_WIDTH / 2, HM_HEIGHT - 1}, "the map is not normalized !", WHITE_, std::nullopt, TCOD_CENTER);
+  }
   // message
-  msgDelay -= TCODSystem::getLastFrameLength();
-  if (msg[0] != 0 && msgDelay > 0.0f) {
-    int h = TCODConsole::root->printRectEx(
-        HM_WIDTH / 2, HM_HEIGHT / 2 + 1, HM_WIDTH / 2 - 2, 0, TCOD_BKGND_NONE, TCOD_CENTER, msg);
-    TCODConsole::root->setDefaultBackground(TCODColor::lightBlue);
-    if (h > 0) TCODConsole::root->rect(HM_WIDTH / 4, HM_HEIGHT / 2, HM_WIDTH / 2, h + 2, false, TCOD_BKGND_SET);
-    TCODConsole::root->setDefaultBackground(TCODColor::black);
+  msgDelay -= delta_time;
+  if (msg.size() && msgDelay > 0.0f) {
+    const int msg_height = tcod::print_rect(
+        console, {HM_WIDTH / 4 + 1, HM_HEIGHT / 2 + 1, HM_WIDTH / 2 - 1, 0}, msg, WHITE_, std::nullopt, TCOD_CENTER);
+    tcod::draw_rect(console, {HM_WIDTH / 4, HM_HEIGHT / 2, HM_WIDTH / 2, msg_height + 2}, 0, std::nullopt, LIGHT_BLUE_);
   }
 }
 
-void message(float delay, const char* fmt, ...) {
-  va_list ap;
+template <typename... T>
+void message(float delay, const char* fmt, T... args) {
   msgDelay = delay;
-  va_start(ap, fmt);
-  vsprintf(msg, fmt, ap);
-  va_end(ap);
+  msg = tcod::stringf(fmt, args...);
 }
 
 void backup() {
   // save the heightmap & RNG states
-  for (int x = 0; x < HM_WIDTH; x++) {
-    for (int y = 0; y < HM_HEIGHT; y++) {
-      hmold->setValue(x, y, hm->getValue(x, y));
+  for (int y = 0; y < HM_HEIGHT; ++y) {
+    for (int x = 0; x < HM_WIDTH; ++x) {
+      hm_old->setValue(x, y, hm->getValue(x, y));
     }
   }
   if (backupRnd) delete backupRnd;
@@ -169,9 +166,9 @@ void backup() {
 
 void restore() {
   // restore the previously saved heightmap & RNG states
-  for (int x = 0; x < HM_WIDTH; x++) {
-    for (int y = 0; y < HM_HEIGHT; y++) {
-      hm->setValue(x, y, hmold->getValue(x, y));
+  for (int y = 0; y < HM_HEIGHT; ++y) {
+    for (int x = 0; x < HM_WIDTH; ++x) {
+      hm->setValue(x, y, hm_old->getValue(x, y));
     }
   }
   rnd->restore(backupRnd);
@@ -190,19 +187,19 @@ void load() {
 }
 
 void addHill(int nbHill, float baseRadius, float radiusVar, float height) {
-  for (int i = 0; i < nbHill; i++) {
-    float hillMinRadius = baseRadius * (1.0f - radiusVar);
-    float hillMaxRadius = baseRadius * (1.0f + radiusVar);
-    float radius = rnd->getFloat(hillMinRadius, hillMaxRadius);
-    float theta = rnd->getFloat(0.0f, 6.283185f);  // between 0 and 2Pi
-    float dist = rnd->getFloat(0.0f, (float)MIN(HM_WIDTH, HM_HEIGHT) / 2 - radius);
-    int xh = (int)(HM_WIDTH / 2 + cos(theta) * dist);
-    int yh = (int)(HM_HEIGHT / 2 + sin(theta) * dist);
+  for (int i = 0; i < nbHill; ++i) {
+    const float hillMinRadius = baseRadius * (1.0f - radiusVar);
+    const float hillMaxRadius = baseRadius * (1.0f + radiusVar);
+    const float radius = rnd->getFloat(hillMinRadius, hillMaxRadius);
+    const float theta = rnd->getFloat(0.0f, 6.283185f);  // between 0 and 2Pi
+    const float dist = rnd->getFloat(0.0f, (float)MIN(HM_WIDTH, HM_HEIGHT) / 2 - radius);
+    const int xh = (int)(HM_WIDTH / 2 + cos(theta) * dist);
+    const int yh = (int)(HM_HEIGHT / 2 + sin(theta) * dist);
     hm->addHill((float)xh, (float)yh, radius, height);
   }
 }
 
-void clearCbk(Widget* w, void* data) {
+void clearCbk(Widget*, void*) {
   hm->clear();
   Operation::clear();
   history->clear();
@@ -210,66 +207,66 @@ void clearCbk(Widget* w, void* data) {
   params->setVisible(false);
 }
 
-void reseedCbk(Widget* w, void* data) {
+void reseedCbk(Widget*, void*) {
   seed = rnd->getInt(0x7FFFFFFF, 0xFFFFFFFF);
   Operation::reseed();
   message(3.0f, "Switching to seed %X", seed);
 }
 
-void cancelCbk(Widget* w, void* data) { Operation::cancel(); }
+void cancelCbk(Widget*, void*) { Operation::cancel(); }
 
 // operations buttons callbacks
-void normalizeCbk(Widget* w, void* data) { (new NormalizeOperation(0.0f, 1.0f))->add(); }
+void normalizeCbk(Widget*, void*) { (new NormalizeOperation(0.0f, 1.0f))->add(); }
 
-void addFbmCbk(Widget* w, void* data) { (new AddFbmOperation(1.0f, addFbmDelta, 0.0f, 6.0f, 1.0f, 0.5f))->add(); }
+void addFbmCbk(Widget*, void*) { (new AddFbmOperation(1.0f, addFbmDelta, 0.0f, 6.0f, 1.0f, 0.5f))->add(); }
 
-void scaleFbmCbk(Widget* w, void* data) { (new ScaleFbmOperation(1.0f, addFbmDelta, 0.0f, 6.0f, 1.0f, 0.5f))->add(); }
+void scaleFbmCbk(Widget*, void*) { (new ScaleFbmOperation(1.0f, addFbmDelta, 0.0f, 6.0f, 1.0f, 0.5f))->add(); }
 
-void addHillCbk(Widget* w, void* data) {
+void addHillCbk(Widget*, void*) {
   (new AddHillOperation(25, 10.0f, 0.5f, (mapmax == mapmin ? 0.5f : (mapmax - mapmin) * 0.1f)))->add();
 }
 
-void rainErosionCbk(Widget* w, void* data) { (new RainErosionOperation(1000, 0.05f, 0.05f))->add(); }
+void rainErosionCbk(Widget*, void*) { (new RainErosionOperation(1000, 0.05f, 0.05f))->add(); }
 
-void smoothCbk(Widget* w, void* data) { (new SmoothOperation(mapmin, mapmax, 2))->add(); }
+void smoothCbk(Widget*, void*) { (new SmoothOperation(mapmin, mapmax, 2))->add(); }
 
-void voronoiCbk(Widget* w, void* data) { (new VoronoiOperation(100, 2, voronoiCoef))->add(); }
+void voronoiCbk(Widget*, void*) { (new VoronoiOperation(100, 2, voronoiCoef))->add(); }
 
-void noiseLerpCbk(Widget* w, void* data) {
+void noiseLerpCbk(Widget*, void*) {
   float v = (mapmax - mapmin) * 0.5f;
   if (v == 0.0f) v = 1.0f;
   (new NoiseLerpOperation(0.0f, 1.0f, addFbmDelta, 0.0f, 6.0f, v, v))->add();
 }
 
-void raiseLowerCbk(Widget* w, void* data) { (new AddLevelOperation(0.0f))->add(); }
+void raiseLowerCbk(Widget*, void*) { (new AddLevelOperation(0.0f))->add(); }
 
 // In/Out buttons callbacks
 
-void exportCCbk(Widget* w, void* data) {
-  const char* code = Operation::buildCode(Operation::C);
-  FILE* f = fopen("hm.c", "wt");
-  fprintf(f, "%s", code);
-  fclose(f);
+void exportCCbk(Widget*, void*) {
+  const std::string code = Operation::buildCode(Operation::C);
+  auto f = std::ofstream("hm.c", std::ios_base::trunc);
+  f << code;
+  f.close();
   message(3.0f, "The code has been exported to ./hm.c");
 }
 
-void exportPyCbk(Widget* w, void* data) {
-  const char* code = Operation::buildCode(Operation::PY);
-  FILE* f = fopen("hm.py", "wt");
-  fprintf(f, "%s", code);
-  fclose(f);
+void exportPyCbk(Widget*, void*) {
+  const std::string code = Operation::buildCode(Operation::PY);
+  auto f = std::ofstream("hm.py", std::ios_base::trunc);
+  f << code;
+  f.close();
   message(3.0f, "The code has been exported to ./hm.py");
 }
 
-void exportCppCbk(Widget* w, void* data) {
-  const char* code = Operation::buildCode(Operation::CPP);
-  FILE* f = fopen("hm.cpp", "wt");
-  fprintf(f, "%s", code);
-  fclose(f);
+void exportCppCbk(Widget*, void*) {
+  const std::string code = Operation::buildCode(Operation::CPP);
+  auto f = std::ofstream("hm.cpp", std::ios_base::trunc);
+  f << code;
+  f.close();
   message(3.0f, "The code has been exported to ./hm.cpp");
 }
 
-void exportBmpCbk(Widget* w, void* data) {
+void exportBmpCbk(Widget*, void*) {
   TCODImage img(HM_WIDTH, HM_HEIGHT);
   for (int x = 0; x < HM_WIDTH; x++) {
     for (int y = 0; y < HM_HEIGHT; y++) {
@@ -305,97 +302,95 @@ void exportBmpCbk(Widget* w, void* data) {
 }
 
 // Display buttons callbacks
-void colorMapCbk(Widget* w, void* data) {
+void colorMapCbk(Widget*, void*) {
   slope = false;
   greyscale = false;
   normal = false;
 }
 
-void greyscaleCbk(Widget* w, void* data) {
+void greyscaleCbk(Widget*, void*) {
   slope = false;
   greyscale = true;
   normal = false;
 }
 
-void slopeCbk(Widget* w, void* data) {
+void slopeCbk(Widget*, void*) {
   slope = true;
   greyscale = false;
   normal = false;
 }
 
-void normalCbk(Widget* w, void* data) {
+void normalCbk(Widget*, void*) {
   slope = false;
   greyscale = false;
   normal = true;
 }
 
-void changeColorMapIdxCbk(Widget* w, float val, void* data) {
-  intptr_t i = ((intptr_t)data);
+void changeColorMapIdxCbk(Widget*, float val, void* data) {
+  auto i = reinterpret_cast<intptr_t>(data);
   keyIndex[i] = (int)(val);
   if (i == 1) sandHeight = (float)(i) / 255.0f;
   initColors();
 }
 
-void changeColorMapRedCbk(Widget* w, float val, void* data) {
-  intptr_t i = ((intptr_t)data);
-  keyColor[i].r = (int)(val);
+void changeColorMapRedCbk(Widget*, float val, void* data) {
+  auto i = reinterpret_cast<intptr_t>(data);
+  keyColor[i].r = static_cast<uint8_t>(val);
   keyImages[i]->setBackgroundColor(keyColor[i]);
   initColors();
 }
 
-void changeColorMapGreenCbk(Widget* w, float val, void* data) {
-  intptr_t i = ((intptr_t)data);
-  keyColor[i].g = (int)(val);
+void changeColorMapGreenCbk(Widget*, float val, void* data) {
+  auto i = reinterpret_cast<intptr_t>(data);
+  keyColor[i].g = static_cast<uint8_t>(val);
   keyImages[i]->setBackgroundColor(keyColor[i]);
   initColors();
 }
 
-void changeColorMapBlueCbk(Widget* w, float val, void* data) {
-  intptr_t i = ((intptr_t)data);
-  keyColor[i].b = (int)(val);
+void changeColorMapBlueCbk(Widget*, float val, void* data) {
+  auto i = reinterpret_cast<intptr_t>(data);
+  keyColor[i].b = static_cast<uint8_t>(val);
   keyImages[i]->setBackgroundColor(keyColor[i]);
   initColors();
 }
 
-void changeColorMapEndCbk(Widget* w, void* data) { colorMapGui->setVisible(false); }
+void changeColorMapEndCbk(Widget*, void*) { colorMapGui->setVisible(false); }
 
-void changeColorMapCbk(Widget* w, void* data) {
+void changeColorMapCbk(Widget* w, void*) {
   colorMapGui->move(w->x + w->w + 2, w->y);
   colorMapGui->clear();
-  for (int i = 0; i < nbColorKeys; i++) {
-    char tmp[64];
-    sprintf(tmp, "Color %d", i);
-    colorMapGui->addSeparator(tmp);
-    HBox* hbox = new HBox(0, 0, 0);
-    VBox* vbox = new VBox(0, 0, 0);
-    colorMapGui->addWidget(hbox);
+  for (intptr_t i = 0; i < nbColorKeys; i++) {
+    colorMapGui->addSeparator(tcod::stringf("Color %d", static_cast<int>(i)).c_str());
+    HBox* h_box = new HBox(0, 0, 0);
+    VBox* v_box = new VBox(0, 0, 0);
+    colorMapGui->addWidget(h_box);
     Slider* idxSlider = new Slider(0, 0, 3, 0.0f, 255.0f, "index", "Index of the key in the color map (0-255)");
     idxSlider->setValue((float)keyIndex[i]);
     idxSlider->setFormat("%.0f");
-    idxSlider->setCallback(changeColorMapIdxCbk, (void*)i);
-    vbox->addWidget(idxSlider);
+    idxSlider->setCallback(changeColorMapIdxCbk, reinterpret_cast<void*>(i));
+    v_box->addWidget(idxSlider);
     keyImages[i] = new Image(0, 0, 0, 2);
     keyImages[i]->setBackgroundColor(keyColor[i]);
-    vbox->addWidget(keyImages[i]);
-    hbox->addWidget(vbox);
+    v_box->addWidget(keyImages[i]);
+    h_box->addWidget(v_box);
 
-    vbox = new VBox(0, 0, 0);
-    hbox->addWidget(vbox);
+    v_box = new VBox(0, 0, 0);
+    h_box->addWidget(v_box);
     Slider* redSlider = new Slider(0, 0, 3, 0.0f, 255.0f, "r", "Red component of the color");
     redSlider->setValue((float)keyColor[i].r);
     redSlider->setFormat("%.0f");
-    redSlider->setCallback(changeColorMapRedCbk, (void*)i);
-    vbox->addWidget(redSlider);
+    redSlider->setCallback(changeColorMapRedCbk, reinterpret_cast<void*>(i));
+    v_box->addWidget(redSlider);
     Slider* greenSlider = new Slider(0, 0, 3, 0.0f, 255.0f, "g", "Green component of the color");
     greenSlider->setValue((float)keyColor[i].g);
     greenSlider->setFormat("%.0f");
-    greenSlider->setCallback(changeColorMapGreenCbk, (void*)i);
-    vbox->addWidget(greenSlider);
+    greenSlider->setCallback(changeColorMapGreenCbk, reinterpret_cast<void*>(i));
+    v_box->addWidget(greenSlider);
     Slider* blueSlider = new Slider(0, 0, 3, 0.0f, 255.0f, "b", "Blue component of the color");
     blueSlider->setValue((float)keyColor[i].b);
     blueSlider->setFormat("%.0f");
-    blueSlider->setCallback(changeColorMapBlueCbk, (void*)i);
-    vbox->addWidget(blueSlider);
+    blueSlider->setCallback(changeColorMapBlueCbk, reinterpret_cast<void*>(i));
+    v_box->addWidget(blueSlider);
   }
   colorMapGui->addWidget(new Button("Ok", NULL, changeColorMapEndCbk, NULL));
   colorMapGui->setVisible(true);
@@ -429,13 +424,13 @@ void buildGui() {
   // operations
   tools->addSeparator("Operations", "Apply a new operation to the map");
   addOperationButton(tools, Operation::NORM, normalizeCbk);
-  addOperationButton(tools, Operation::ADDFBM, addFbmCbk);
-  addOperationButton(tools, Operation::SCALEFBM, scaleFbmCbk);
+  addOperationButton(tools, Operation::ADD_FBM, addFbmCbk);
+  addOperationButton(tools, Operation::SCALE_FBM, scaleFbmCbk);
   addOperationButton(tools, Operation::ADDHILL, addHillCbk);
   addOperationButton(tools, Operation::RAIN, rainErosionCbk);
   addOperationButton(tools, Operation::SMOOTH, smoothCbk);
   addOperationButton(tools, Operation::VORONOI, voronoiCbk);
-  addOperationButton(tools, Operation::NOISELERP, noiseLerpCbk);
+  addOperationButton(tools, Operation::NOISE_LERP, noiseLerpCbk);
   addOperationButton(tools, Operation::ADDLEVEL, raiseLowerCbk);
 
   // display
@@ -475,27 +470,41 @@ void buildGui() {
 }
 
 int main(int argc, char* argv[]) {
+  TCOD_ContextParams context_params{};
+  context_params.tcod_version = SDL_COMPILEDVERSION;
+  context_params.columns = HM_WIDTH;
+  context_params.rows = HM_HEIGHT;
+  context_params.argc = argc;
+  context_params.argv = argv;
+  context_params.vsync = true;
+  context_params.window_title = "height map tool";
+  context_params.sdl_window_flags = SDL_WINDOW_RESIZABLE;
+
   TCODConsole::initRoot(HM_WIDTH, HM_HEIGHT, "height map tool", false, TCOD_RENDERER_OPENGL2);
   guicon = new TCODConsole(HM_WIDTH, HM_HEIGHT);
   guicon->setKeyColor(TCODColor(255, 0, 255));
   Widget::setConsole(guicon);
-  TCODSystem::setFps(25);
+  int desired_fps = 25;
   initColors();
   buildGui();
   hm = new TCODHeightMap(HM_WIDTH, HM_HEIGHT);
-  hmold = new TCODHeightMap(HM_WIDTH, HM_HEIGHT);
+  hm_old = new TCODHeightMap(HM_WIDTH, HM_HEIGHT);
   rnd = new TCODRandom(seed);
   noise = new TCODNoise(2, rnd);
   uint8_t fade = 50;
   bool creditsEnd = false;
+  auto timer = tcod::Timer();
 
-  while (!TCODConsole::isWindowClosed()) {
-    render();
+  while (true) {
+    auto context = TCOD_sys_get_internal_context();
+    auto console = TCOD_sys_get_internal_console();
+    const float delta_time = timer.sync(desired_fps);
+    render(*console, delta_time);
     guicon->setDefaultBackground(TCODColor(255, 0, 255));
     guicon->clear();
     Widget::renderWidgets();
     if (!creditsEnd) {
-      creditsEnd = TCODConsole::renderCredits(HM_WIDTH - 20, HM_HEIGHT - 7, true);
+      creditsEnd = TCOD_console_credits_render_ex(console, HM_WIDTH - 20, HM_HEIGHT - 7, true, delta_time);
     }
     if (Widget::focus) {
       if (fade < 200) fade += 20;
@@ -503,29 +512,38 @@ int main(int argc, char* argv[]) {
       if (fade > 80) fade -= 20;
     }
     TCODConsole::blit(guicon, 0, 0, HM_WIDTH, HM_HEIGHT, TCODConsole::root, 0, 0, fade / 255.0f, fade / 255.0f);
-    TCODConsole::flush();
-    TCOD_key_t key;
-    TCOD_mouse_t mouse;
-    TCODSystem::checkForEvent(TCOD_EVENT_KEY_PRESS | TCOD_EVENT_MOUSE, &key, &mouse);
-    Widget::updateWidgets(key, mouse);
-    switch (key.c) {
-      case '+':
-        (new AddLevelOperation((mapmax - mapmin) / 50))->run();
-        break;
-      case '-':
-        (new AddLevelOperation(-(mapmax - mapmin) / 50))->run();
-        break;
-      default:
-        break;
-    }
-    switch (key.vk) {
-      case TCODK_PRINTSCREEN:
-        TCODSystem::saveScreenshot(NULL);
-        break;
-      default:
-        break;
+    context->present(*console);
+
+    SDL_Event event;
+    while (SDL_PollEvent(&event)) {
+      TCOD_key_t key{};
+      TCOD_mouse_t mouse{};
+      tcod::sdl2::process_event(event, key);
+      tcod::sdl2::process_event(event, mouse);
+      Widget::updateWidgets(key, mouse);
+      switch (event.type) {
+        case SDL_QUIT:
+          std::exit(EXIT_SUCCESS);
+          break;
+        case SDL_KEYDOWN:
+          switch (event.key.keysym.sym) {
+            case SDLK_MINUS:
+              (new AddLevelOperation(-(mapmax - mapmin) / 50))->run();
+              break;
+            case SDLK_EQUALS:
+              (new AddLevelOperation((mapmax - mapmin) / 50))->run();
+              break;
+            case SDLK_PRINTSCREEN:
+              context->save_screenshot(nullptr);
+              break;
+            default:
+              break;
+          }
+          break;
+        default:
+          break;
+      }
     }
   }
-
   return 0;
 }
