@@ -25,46 +25,46 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <libtcod.h>
+#include <libtcod.hpp>
 
 #include "rad_shader.hpp"
 
 void StandardShader::compute() {
   // turn off all lights
-  memset(lightmap, 0, sizeof(TCODColor) * map->getWidth() * map->getHeight());
-  for (const Light& l : lights) {
+  memset(lightmap_, 0, sizeof(TCODColor) * map_->getWidth() * map_->getHeight());
+  for (const Light& l : lights_) {
     // compute the potential visible set for this light
     int minx = l.x - l.radius;
     int miny = l.y - l.radius;
     int maxx = l.x + l.radius;
     int maxy = l.y + l.radius;
-    minx = MAX(0, minx);
-    miny = MAX(0, miny);
-    maxx = MIN(maxx, map->getWidth() - 1);
-    maxy = MIN(maxy, map->getHeight() - 1);
-    float offset = 1.0f / (1.0f + (float)(l.radius * l.radius) / 20);
-    float factor = 1.0f / (1.0f - offset);
+    miny = std::max(0, miny);
+    minx = std::max(0, minx);
+    maxx = std::min(maxx, map_->getWidth() - 1);
+    maxy = std::min(maxy, map_->getHeight() - 1);
+    const float offset = 1.0f / (1.0f + static_cast<float>(l.radius * l.radius) / 20.0f);
+    const float factor = 1.0f / (1.0f - offset);
     // compute the light's fov
-    TCODMap lmap(maxx - minx + 1, maxy - miny + 1);
+    TCODMap l_map(maxx - minx + 1, maxy - miny + 1);
     for (int x = minx; x <= maxx; x++) {
       for (int y = miny; y <= maxy; y++) {
-        lmap.setProperties(x - minx, y - miny, map->isWalkable(x, y), map->isTransparent(x, y));
+        l_map.setProperties(x - minx, y - miny, map_->isWalkable(x, y), map_->isTransparent(x, y));
       }
     }
-    lmap.computeFov(l.x - minx, l.y - miny, l.radius);
+    l_map.computeFov(l.x - minx, l.y - miny, l.radius);
     // compute the light's contribution
     // double invSquaredRadius=1.0 / (l->radius*l->radius);
     for (int x = minx; x <= maxx; x++) {
       for (int y = miny; y <= maxy; y++) {
-        if (lmap.isInFov(x - minx, y - miny)) {
-          int squaredDist = (l.x - x) * (l.x - x) + (l.y - y) * (l.y - y);
+        if (l_map.isInFov(x - minx, y - miny)) {
+          const int squaredDist = (l.x - x) * (l.x - x) + (l.y - y) * (l.y - y);
           // basic
           // double coef = 1.0-squaredDist*invSquaredRadius;
-          // invsqr1
+          // inv_sqr1
           // double coef=(1.0f/(1.0f+(float)(squaredDist)));
-          // invsqr2
-          float coef = (1.0f / (1.0f + (float)(squaredDist) / 20) - offset) * factor;
-          TCODColor* col = &lightmap[x + y * map->getWidth()];
+          // inv_sqr2
+          const float coef = (1.0f / (1.0f + static_cast<float>(squaredDist) / 20.0f) - offset) * factor;
+          TCODColor* col = &lightmap_[x + y * map_->getWidth()];
           *col = *col + l.col * coef;
         }
       }
