@@ -23,23 +23,10 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
-#include <libtcod.h>
-
-#include "util_ripples.hpp"
-
-#define VERSION "0.1.0"
+#include <libtcod.hpp>
 
 // console size
-#define CON_W 80
-#define CON_H 50
-
-#define IN_RECTANGLE(x, y, w, h) ((unsigned)(x) < (unsigned)(w) && (unsigned)(y) < (unsigned)(h))
-#define SQRDIST(x1, y1, x2, y2) (((x1) - (x2)) * ((x1) - (x2)) + ((y1) - (y2)) * ((y1) - (y2)))
-
-#ifndef NDEBUG
-#define DBG(x) printf x
-#else
-#define DBG(x)
-#endif
+static constexpr auto CON_W = 80;
+static constexpr auto CON_H = 50;
 
 extern TCODNoise noise3d;
