@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,10 +41,10 @@ extern "C" {
 #endif
 typedef struct {
   TCOD_tree_t tree; /* pseudo oop : bsp inherit tree */
-  int x, y, w, h;   /* node position & size */
-  int position;     /* position of splitting */
-  uint8_t level;    /* level in the tree */
-  bool horizontal;  /* horizontal splitting ? */
+  int x, y, w, h; /* node position & size */
+  int position; /* position of splitting */
+  uint8_t level; /* level in the tree */
+  bool horizontal; /* horizontal splitting ? */
 } TCOD_bsp_t;
 
 typedef bool (*TCOD_bsp_callback_t)(TCOD_bsp_t* node, void* userData);
@@ -68,7 +68,7 @@ TCODLIB_API TCOD_bsp_t* TCOD_bsp_find_node(TCOD_bsp_t* node, int x, int y);
 TCODLIB_API void TCOD_bsp_resize(TCOD_bsp_t* node, int x, int y, int w, int h);
 TCODLIB_API void TCOD_bsp_split_once(TCOD_bsp_t* node, bool horizontal, int position);
 TCODLIB_API void TCOD_bsp_split_recursive(
-    TCOD_bsp_t* node, TCOD_random_t randomizer, int nb, int minHSize, int minVSize, float maxHRatio, float maxVRatio);
+    TCOD_bsp_t* node, TCOD_Random* randomizer, int nb, int minHSize, int minVSize, float maxHRatio, float maxVRatio);
 TCODLIB_API void TCOD_bsp_remove_sons(TCOD_bsp_t* node);
 #ifdef __cplusplus
 }
