@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -157,7 +157,7 @@ void TCOD_bsp_split_once(TCOD_bsp_t* node, bool horizontal, int position) {
 }
 
 void TCOD_bsp_split_recursive(
-    TCOD_bsp_t* node, TCOD_random_t randomizer, int nb, int minHSize, int minVSize, float maxHRatio, float maxVRatio) {
+    TCOD_bsp_t* node, TCOD_Random* randomizer, int nb, int minHSize, int minVSize, float maxHRatio, float maxVRatio) {
   bool horiz;
   int position;
   if (nb == 0 || (node->w < 2 * minHSize && node->h < 2 * minVSize)) return;
