@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,9 +34,14 @@
 
 #include <stdbool.h>
 #ifdef __cplusplus
+#include <algorithm>
 #include <array>
 #include <memory>
+#include <optional>
 #include <stdexcept>
+#include <utility>
+
+#include "error.hpp"
 #endif  // __cplusplus
 
 #include "color.h"
@@ -70,89 +75,138 @@ typedef enum {
  *  Print justification options.
  */
 typedef enum { TCOD_LEFT, TCOD_RIGHT, TCOD_CENTER } TCOD_alignment_t;
-/**
-    A console tile.
+/***************************************************************************
+    @brief The raw data for a single TCOD_Console tile.
+
+    \rst
+    .. versionadded:: 1.19
+    \endrst
  */
 typedef struct TCOD_ConsoleTile {
 #ifdef __cplusplus
   bool operator==(const TCOD_ConsoleTile& rhs) const noexcept { return ch == rhs.ch && fg == rhs.fg && bg == rhs.bg; }
   bool operator!=(const TCOD_ConsoleTile& rhs) const noexcept { return !(*this == rhs); }
 #endif  // __cplusplus
-  /**
-      The Unicode codepoint for this tile.
+  /***************************************************************************
+      @brief The Unicode codepoint for this tile.
    */
   int ch;
-  /**
-      The tile glyph color, rendered on top of the background.
+  /***************************************************************************
+      @brief The tile glyph color, rendered on top of the background.
    */
   TCOD_ColorRGBA fg;
-  /**
-      The tile background color, rendered behind the glyph.
+  /***************************************************************************
+      @brief The tile background color, rendered behind the glyph.
    */
   TCOD_ColorRGBA bg;
 } TCOD_ConsoleTile;
-/**
-    The libtcod console struct.
+/***************************************************************************
+    @brief A libtcod console containing a grid of tiles with `{ch, fg, bg}` information.
 
-    All attributes should be considered private.
+    @details In C++ this struct has several convience methods to make working with consoles easier.
+    Note that all tile references are to TCOD_ConsoleTile structs and will include an alpha channel.
 
-    All C++ methods should be considered provisional, and are subject to
-    change.
+    For C++ code examples see `tcod::Console`.
+
+    \rst
+    .. versionadded:: 1.19
+    \endrst
  */
 struct TCOD_Console {
 #ifdef __cplusplus
-  auto begin() noexcept -> TCOD_ConsoleTile* { return tiles; }
-  auto begin() const noexcept -> const TCOD_ConsoleTile* { return tiles; }
-  auto end() noexcept -> TCOD_ConsoleTile* { return tiles + size(); }
-  auto end() const noexcept -> const TCOD_ConsoleTile* { return tiles + size(); }
-  /**
-      Return a reference to the tile at `xy`.
+  /***************************************************************************
+      @brief Return a pointer to the beginning of this consoles tile data.
    */
-  auto operator[](const std::array<int, 2>& xy) noexcept -> TCOD_ConsoleTile& { return tiles[w * xy[1] + xy[0]]; }
-  /**
-      Return a constant reference to the tile at `xy`.
+  [[nodiscard]] auto begin() noexcept -> TCOD_ConsoleTile* { return tiles; }
+  /***************************************************************************
+      @brief Return a const pointer to the beginning of this consoles tile data.
    */
-  auto operator[](const std::array<int, 2>& xy) const noexcept -> const TCOD_ConsoleTile& {
-    return tiles[w * xy[1] + xy[0]];
-  }
-  /**
-      Return a reference to the tile at `x`,`y`.
+  [[nodiscard]] auto begin() const noexcept -> const TCOD_ConsoleTile* { return tiles; }
+  /***************************************************************************
+      @brief Return a pointer to the end of this consoles tile data.
+   */
+  [[nodiscard]] auto end() noexcept -> TCOD_ConsoleTile* { return tiles + elements; }
+  /***************************************************************************
+      @brief Return a const pointer to the end of this consoles tile data.
+   */
+  [[nodiscard]] auto end() const noexcept -> const TCOD_ConsoleTile* { return tiles + elements; }
+  /***************************************************************************
+      @brief Clear a console by setting all tiles to the provided TCOD_ConsoleTile object.
 
-      Throws if the index is out-of-bounds.
+      @param tile A TCOD_ConsoleTile reference which will be used to clear the console.
    */
-  auto at(int x, int y) -> TCOD_ConsoleTile& {
-    range_check_(x, y);
-    return (*this)[{x, y}];
+  void clear(const TCOD_ConsoleTile& tile = {0x20, {255, 255, 255, 255}, {0, 0, 0, 255}}) noexcept {
+    for (auto& it : *this) it = tile;
   }
-  /**
-      Return a constant reference to the tile at `x`,`y`.
+  /***************************************************************************
+      @brief Return a reference to the tile at `xy`.
+   */
+  [[nodiscard]] auto operator[](const std::array<int, 2>& xy) noexcept -> TCOD_ConsoleTile& {
+    return tiles[get_index(xy)];
+  }
+  /***************************************************************************
+      @brief Return a constant reference to the tile at `xy`.
+   */
+  [[nodiscard]] auto operator[](const std::array<int, 2>& xy) const noexcept -> const TCOD_ConsoleTile& {
+    return tiles[get_index(xy)];
+  }
+  /***************************************************************************
+      @brief Return a reference to the tile at `xy`.
 
-      Throws if the index is out-of-bounds.
+      @throws std::out_of_range if the index is out-of-bounds
    */
-  auto at(int x, int y) const -> const TCOD_ConsoleTile& {
-    range_check_(x, y);
-    return (*this)[{x, y}];
+  [[nodiscard]] auto at(const std::array<int, 2>& xy) -> TCOD_ConsoleTile& { return tiles[bounds_check(xy)]; }
+  /***************************************************************************
+      @brief Return a constant reference to the tile at `xy`.
+
+      @throws std::out_of_range if the index is out-of-bounds
+   */
+  [[nodiscard]] auto at(const std::array<int, 2>& xy) const -> const TCOD_ConsoleTile& {
+    return tiles[bounds_check(xy)];
   }
-  /**
-      Return the total number of tiles in this console.
+  /***************************************************************************
+      @brief Return a reference to the tile at `x`,`y`.
+
+      @throws std::out_of_range if the index is out-of-bounds
    */
-  int size() const { return elements; }
-  /**
-      Internal function.  Throws `std::out_of_range` if `x` or `y` is out-of-bounds.
+  [[nodiscard]] auto at(int x, int y) -> TCOD_ConsoleTile& { return at({x, y}); }
+  /***************************************************************************
+      @brief Return a constant reference to the tile at `x`,`y`.
+
+      @throws std::out_of_range if the index is out-of-bounds
    */
-  void range_check_(int x, int y) const {
-    if (!in_bounds(x, y)) {
+  [[nodiscard]] auto at(int x, int y) const -> const TCOD_ConsoleTile& { return at({x, y}); }
+  /***************************************************************************
+      @brief Convert `xy` into a 1-dimensional index.  Out-of-bounds indexes are undefined.
+
+      @details This index is normally used to index the tiles attribute.
+   */
+  [[nodiscard]] int get_index(const std::array<int, 2>& xy) const noexcept { return w * xy[1] + xy[0]; }
+  /***************************************************************************
+      @brief Return true if `xy` are within the bounds of this console.
+   */
+  [[nodiscard]] bool in_bounds(const std::array<int, 2>& xy) const noexcept {
+    return 0 <= xy[0] && xy[0] < w && 0 <= xy[1] && xy[1] < h;
+  }
+
+ private:
+  /***************************************************************************
+      @brief Checks if `xy` is in bounds then return an in-bounds index.
+
+      @throws std::out_of_range if `xy` is out-of-bounds
+   */
+  int bounds_check(const std::array<int, 2>& xy) const {
+    if (!in_bounds(xy)) {
       throw std::out_of_range(
-          std::string("Out of bounds lookup {x=") + std::to_string(x) + ", y=" + std::to_string(y) +
+          std::string("Out of bounds lookup {") + std::to_string(xy[0]) + ", " + std::to_string(xy[1]) +
           "} on console of shape {" + std::to_string(w) + ", " + std::to_string(h) + "}.");
     }
+    return get_index(xy);
   }
-  /**
-      Return true if `x`,`y` are within the bounds of this console.
-   */
-  bool in_bounds(int x, int y) const noexcept { return 0 <= x && x < w && 0 <= y && y < h; }
+
+ public:
 #endif  // __cplusplus
-  /** Console width and height (in characters, not pixels.) */
+  /** Console width and height in tiles. */
   int w, h;
   /** A contiguous array of console tiles. */
   TCOD_ConsoleTile* __restrict tiles;
@@ -167,7 +221,7 @@ struct TCOD_Console {
   /** The current key color for this console. */
   TCOD_color_t key_color;
   /**
-      The total length of the tiles array.  Same as `w * h`.
+      @brief The total length of the tiles array.  Same as `w * h`.
 
       \rst
       .. versionadded:: 1.16
@@ -175,6 +229,8 @@ struct TCOD_Console {
    */
   int elements;
   /**
+      @brief A userdata attribute which can be repurposed.
+
       \rst
       .. versionadded:: 1.16
       \endrst
@@ -208,14 +264,14 @@ TCOD_PUBLIC void TCOD_console_set_key_color(TCOD_Console* con, TCOD_color_t col)
 /**
  *  Blit from one console to another.
  *
- *  \param srcCon Pointer to the source console.
+ *  \param src Pointer to the source console.
  *  \param xSrc The left region of the source console to blit from.
  *  \param ySrc The top region of the source console to blit from.
  *  \param wSrc The width of the region to blit from.
  *              If 0 then it will fill to the maximum width.
  *  \param hSrc The height of the region to blit from.
  *              If 0 then it will fill to the maximum height.
- *  \param dstCon Pointer to the destination console.
+ *  \param dst Pointer to the destination console.
  *  \param xDst The left corner to blit onto the destination console.
  *  \param yDst The top corner to blit onto the destination console.
  *  \param foreground_alpha Foreground blending alpha.
@@ -253,14 +309,16 @@ TCOD_PUBLIC void TCOD_console_blit_key_color(
 /**
  *  Delete a console.
  *
- *  \param con A console pointer.
+ *  \param console A console pointer.
  *
  *  If the console being deleted is the root console, then the display will be
  *  uninitialized.
  */
 TCOD_PUBLIC void TCOD_console_delete(TCOD_Console* console);
 
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC void TCOD_console_set_default_background(TCOD_Console* con, TCOD_color_t col);
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC void TCOD_console_set_default_foreground(TCOD_Console* con, TCOD_color_t col);
 /**
  *  Clear a console to its default colors and the space character code.
@@ -322,10 +380,12 @@ TCOD_PUBLIC void TCOD_console_put_char_ex(TCOD_Console* con, int x, int y, int c
  *  \param con A console pointer.
  *  \param flag One of `TCOD_bkgnd_flag_t`.
  */
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC void TCOD_console_set_background_flag(TCOD_Console* con, TCOD_bkgnd_flag_t flag);
 /**
  *  Return a consoles default background flag.
  */
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC TCOD_NODISCARD TCOD_bkgnd_flag_t TCOD_console_get_background_flag(TCOD_Console* con);
 /**
  *  Set a consoles default alignment.
@@ -333,13 +393,16 @@ TCOD_PUBLIC TCOD_NODISCARD TCOD_bkgnd_flag_t TCOD_console_get_background_flag(TC
  *  \param con A console pointer.
  *  \param alignment One of TCOD_alignment_t
  */
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC void TCOD_console_set_alignment(TCOD_Console* con, TCOD_alignment_t alignment);
 /**
  *  Return a consoles default alignment.
  */
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC TCOD_NODISCARD TCOD_alignment_t TCOD_console_get_alignment(TCOD_Console* con);
-
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC TCOD_NODISCARD TCOD_color_t TCOD_console_get_default_background(TCOD_Console* con);
+TCOD_DEPRECATED("Console defaults have been deprecated.")
 TCOD_PUBLIC TCOD_NODISCARD TCOD_color_t TCOD_console_get_default_foreground(TCOD_Console* con);
 /**
  *  Return the background color of a console at x,y
@@ -369,14 +432,19 @@ TCOD_PUBLIC TCOD_NODISCARD TCOD_color_t TCOD_console_get_char_foreground(const T
  */
 TCOD_PUBLIC TCOD_NODISCARD int TCOD_console_get_char(const TCOD_Console* con, int x, int y);
 /**
- *  Fade the color of the display.
- *
- *  \param val Where at 255 colors are normal and at 0 colors are completely
- *             faded.
- *  \param fadecol Color to fade towards.
+    Fade the color of the display.
+
+    \param val Where at 255 colors are normal and at 0 colors are completely
+               faded.
+    \param fade_color Color to fade towards.
+    \rst
+    .. deprecated:: 1.19
+        This function will not work with libtcod contexts.
+    \endrst
  */
+TCOD_DEPRECATED("This function does not support contexts.")
 TCOD_PUBLIC
-void TCOD_console_set_fade(uint8_t val, TCOD_color_t fade);
+void TCOD_console_set_fade(uint8_t val, TCOD_color_t fade_color);
 /**
  *  Return the fade value.
  *
@@ -392,19 +460,5 @@ TCOD_PUBLIC TCOD_NODISCARD TCOD_color_t TCOD_console_get_fading_color(void);
 void TCOD_console_resize_(TCOD_Console* console, int width, int height);
 #ifdef __cplusplus
 }  // extern "C"
-namespace tcod {
-struct ConsoleDeleter {
-  void operator()(TCOD_Console* console) const { TCOD_console_delete(console); }
-};
-typedef std::unique_ptr<struct TCOD_Console, ConsoleDeleter> ConsolePtr;
-TCOD_NODISCARD
-inline auto new_console(int width, int height) -> ConsolePtr {
-  ConsolePtr console{TCOD_console_new(width, height)};
-  if (!console) {
-    throw std::runtime_error(TCOD_get_error());
-  }
-  return console;
-}
-}  // namespace tcod
 #endif  // __cplusplus
 #endif  // TCOD_CONSOLE_H_
