@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,21 +29,56 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-// clang-format off
 #ifndef _TCOD_CONSOLE_HPP
 #define _TCOD_CONSOLE_HPP
 
+#include <stdexcept>
 #include <string>
+#include <utility>
 
+#include "color.hpp"
 #include "console.h"
+#include "console_drawing.h"
 #include "console_etc.h"
 #include "console_init.h"
-#include "console_drawing.h"
 #include "console_printing.h"
 #include "console_rexpaint.h"
-
+#include "console_types.h"
 #include "image.hpp"
-#include "color.hpp"
+
+namespace tcod {
+/***************************************************************************
+    @brief Blit a region of tiles from one console to another.
+
+    @param dest The destination console.
+    @param source The source console to blit from.
+    @param dest_xy The upper-left position of the destination console to blit to.
+    @param source_rect The source region `{left, top, width, height}` to blit from.
+        A width or height of zero will use the entire console.
+    @param foreground_alpha
+    @param background_alpha
+ */
+inline void blit(
+    TCOD_Console& dest,
+    const TCOD_Console& source,
+    const std::array<int, 2>& dest_xy = {0, 0},
+    std::array<int, 4> source_rect = {0, 0, 0, 0},
+    float foreground_alpha = 1.0f,
+    float background_alpha = 1.0f) {
+  TCOD_console_blit(
+      &source,
+      source_rect.at(0),
+      source_rect.at(1),
+      source_rect.at(2),
+      source_rect.at(3),
+      &dest,
+      dest_xy.at(0),
+      dest_xy.at(1),
+      foreground_alpha,
+      background_alpha);
+}
+}  // namespace tcod
+// clang-format off
 
 class TCODImage;
 /**
@@ -90,8 +125,18 @@ Classic real time game loop:
 		}
 */
 
+// clang-format on
 class TCODLIB_API TCODConsole {
-public :
+ public:
+  /***************************************************************************
+      @brief Default constructor.
+
+      \rst
+      .. versionadded:: 1.24
+      \endrst
+   */
+  TCODConsole();
+  // clang-format off
 	/**
 	@PageName console_init
 	@PageTitle Initializing the console
@@ -137,6 +182,7 @@ public :
 	@PyEx libtcod.console_init_root(80, 50, 'The Chronicles Of Doryen v0.1')
 	@LuaEx tcod.console.initRoot(80,50,"The Chronicles Of Doryen v0.1")
 	*/
+  [[deprecated("This way of initializing libtcod is deprecated.  See the documentation for how to use tcod::new_context.")]]
 	static void initRoot(int w, int h, const char * title, bool fullscreen = false, TCOD_renderer_t renderer=TCOD_RENDERER_SDL);
 
 	/**
@@ -213,8 +259,14 @@ public :
 		tcod.console.setCustomFont("32bits_8x8_ascii_in_row_font.png",tcod.LayoutAsciiInRow);
 		tcod.console.setCustomFont("greyscale_8x8_tcod_font.png",tcod.LayoutTCOD + tcod.Greyscale);
 	*/
-	static void setCustomFont(const char *fontFile, int flags=TCOD_FONT_LAYOUT_ASCII_INCOL,int nbCharHoriz=0, int nbCharVertic=0);
-
+  // clang-format on
+  [[deprecated(
+      "This function is not compatible with contexts."
+      "  Consider using tcod::load_tilesheet or TCOD_tileset_load instead."
+      "  https://libtcod.readthedocs.io/en/latest/upgrading.html")]] static void
+  setCustomFont(
+      const char* fontFile, int flags = TCOD_FONT_LAYOUT_ASCII_INCOL, int nbCharHoriz = 0, int nbCharVertic = 0);
+  // clang-format off
 	/**
 	@PageName console_map
 	@PageTitle Using custom characters mapping
@@ -231,6 +283,7 @@ public :
 	@Param asciiCode ASCII code to map.
 	@Param fontCharX,fontCharY Coordinate of the character in the bitmap font (in characters, not pixels).
 	*/
+  [[deprecated("This function is not compatible with contexts.")]]
 	static void mapAsciiCodeToFont(int asciiCode, int fontCharX, int fontCharY);
 
 	/**
@@ -245,6 +298,7 @@ public :
 	@Param nbCodes number of consecutive ASCII codes to map
 	@Param fontCharX,fontCharY coordinate of the character in the bitmap font (in characters, not pixels) corresponding to the first ASCII code
 	*/
+  [[deprecated("This function is not compatible with contexts.")]]
 	static void mapAsciiCodesToFont(int firstAsciiCode, int nbCodes, int fontCharX, int fontCharY);
 
 	/**
@@ -258,6 +312,7 @@ public :
 	@Param s string containing the ASCII codes to map
 	@Param fontCharX,fontCharY coordinate of the character in the bitmap font (in characters, not pixels) corresponding to the first ASCII code in the string
 	*/
+  [[deprecated("This function is not compatible with contexts.")]]
 	static void mapStringToFont(const char *s, int fontCharX, int fontCharY);
 
 	/**
@@ -272,6 +327,7 @@ public :
 	@C# static bool TCODConsole::isFullscreen()
 	@Lua tcod.console.isFullscreen()
 	*/
+  [[deprecated("This function is not compatible with contexts. Use SDL_GetWindowFlags to check this.")]]
 	static bool isFullscreen();
 	/**
 	@PageName console_fullscreen
@@ -306,6 +362,7 @@ public :
 			tcod.console.setFullscreen(not tcod.console.isFullscreen())
 		end
 	*/
+  [[deprecated("This function is not compatible with contexts. Use SDL_SetWindowFullscreen to set this state.")]]
 	static void setFullscreen(bool fullscreen);
 
 	/**
@@ -322,6 +379,7 @@ public :
 	@Lua tcod.console.setWindowTitle(title)
 	@Param title New title of the game window
 	*/
+  [[deprecated("This function is not compatible with contexts. Use SDL_SetWindowTitle to change the window title.")]]
 	static void setWindowTitle(const char *title);
 
 	/**
@@ -334,6 +392,7 @@ public :
 	@C# static bool TCODConsole::isWindowClosed()
 	@Lua tcod.console.isWindowClosed()
 	*/
+  [[deprecated("This function is not compatible with contexts. Use SDL for events and check for SDL_QUIT.")]]
 	static bool isWindowClosed();
 
 	/**
@@ -344,6 +403,7 @@ public :
 	@C bool TCOD_console_has_mouse_focus()
 	@Py console_has_mouse_focus()
 	*/
+  [[deprecated("This function is not compatible with contexts. Use SDL_GetWindowFlags to check this.")]]
 	static bool hasMouseFocus();
 
 	/**
@@ -354,6 +414,7 @@ public :
 	@C bool TCOD_console_is_active()
 	@Py console_is_active()
 	*/
+  [[deprecated("This function is not compatible with contexts. Use SDL_GetWindowFlags to check this.")]]
 	static bool isActive();
 
 	/**
@@ -370,6 +431,7 @@ public :
 	@C# static void TCODConsole::credits()
 	@Lua tcod.console.credits()
 	*/
+  [[deprecated("This function is not compatible with contexts. Consider using `TCOD_console_credits_render_ex`.")]]
 	static void credits();
 
 	/**
@@ -421,6 +483,7 @@ public :
 			tcod.console.flush()
 		end
 	*/
+  [[deprecated("This function is not compatible with contexts. Consider using `TCOD_console_credits_render_ex`.")]]
 	static bool renderCredits(int x, int y, bool alpha);
 
 	/**
@@ -433,6 +496,7 @@ public :
 	@C# static void TCODConsole::resetCredits()
 	@Lua tcod.console.resetCredits()
 	*/
+  [[deprecated]]
 	static void resetCredits();
 
 	/**
@@ -459,6 +523,7 @@ public :
 	@PyEx libtcod.console_set_default_background(0, my_color)
 	@Lua libtcod.TCODConsole_root:setBackgroundColor( myColor )
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	void setDefaultBackground(TCODColor back);
 
 	/**
@@ -477,6 +542,7 @@ public :
 	@PyEx libtcod.console_set_default_foreground(0, my_color)
 	@LuaEx libtcod.TCODConsole_root:setForegroundColor( myColor )
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	void setDefaultForeground(TCODColor fore);
 
 	/**
@@ -515,6 +581,7 @@ public :
 	@Param col the background color to use. You can use color constants
 	@Param flag this flag defines how the cell's background color is modified. See <a href="console_bkgnd_flag_t.html">TCOD_bkgnd_flag_t</a>
 	*/
+  [[deprecated]]
 	void setCharBackground(int x, int y, const TCODColor &col, TCOD_bkgnd_flag_t flag = TCOD_BKGND_SET);
 	/**
 	@PageName console_draw_basic
@@ -531,13 +598,14 @@ public :
 		0 <= y < console height
 	@Param col the foreground color to use. You can use color constants
 	*/
+  [[deprecated]]
 	void setCharForeground(int x, int y, const TCODColor &col);
 
 	/**
 	@PageName console_draw_basic
 	@FuncTitle Setting the ASCII code of a cell
 	@FuncDesc This function modifies the ASCII code of a cell, leaving other properties (background and foreground colors) unchanged.
-		Note that since a clear console has both background and foreground colors set to black for every cell, using setchar will produce black characters on black background. Use putchar instead.
+		Note that since a clear console has both background and foreground colors set to black for every cell, using setChar will produce black characters on black background. Use putchar instead.
 	@Cpp void TCODConsole::setChar(int x, int y, int c)
 	@C void TCOD_console_set_char(TCOD_console_t con,int x, int y, int c)
 	@Py console_set_char(con, x,  y, c)
@@ -549,6 +617,7 @@ public :
 		0 <= y < console height
 	@Param c the new ASCII code for the cell. You can use ASCII constants
 	*/
+  [[deprecated]]
 	void setChar(int x, int y, int c);
 
 	/**
@@ -574,6 +643,7 @@ public :
 	@Param c the new ASCII code for the cell. You can use ASCII constants
 	@Param flag this flag defines how the cell's background color is modified. See TCOD_bkgnd_flag_t
 	*/
+  [[deprecated]]
 	void putChar(int x, int y, int c, TCOD_bkgnd_flag_t flag = TCOD_BKGND_DEFAULT);
 
 	/**
@@ -595,6 +665,7 @@ public :
 	@Param c the new ASCII code for the cell. You can use ASCII constants
 	@Param fore,back new foreground and background colors for this cell
 	*/
+  [[deprecated]]
 	void putCharEx(int x, int y, int c, const TCODColor &fore, const TCODColor &back);
 
 	/**
@@ -637,6 +708,7 @@ public :
 	@Param con in the C and Python versions, the offscreen console handler or NULL for the root console
 	@Param flag this flag defines how the cell's background color is modified. See TCOD_bkgnd_flag_t
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	void setBackgroundFlag(TCOD_bkgnd_flag_t flag);
 
 	/**
@@ -651,6 +723,7 @@ public :
 	@Lua Console:getBackgroundFlag()
 	@Param con in the C and Python versions, the offscreen console handler or NULL for the root console
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	TCOD_bkgnd_flag_t getBackgroundFlag() const;
 
 	/**
@@ -668,6 +741,7 @@ public :
 	@Param con in the C and Python versions, the offscreen console handler or NULL for the root console
 	@Param alignment defines how the strings are printed on screen.
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	void setAlignment(TCOD_alignment_t alignment);
 
 	/**
@@ -684,6 +758,7 @@ public :
 	@Lua Console:getAlignment()
 	@Param con in the C and Python versions, the offscreen console handler or NULL for the root console
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	TCOD_alignment_t getAlignment() const;
   /**
    *  Print an EASCII formatted string to the console.
@@ -693,10 +768,10 @@ public :
    *    UTF-8 overloads.
    *  \endrst
    */
-  TCOD_DEPRECATED(
-    "Use TCODConsole::printf or the std::string overload for this function."
-    "\nNote that you'll need to use UTF-8 encoded strings for those functions.")
+  [[deprecated("Use TCODConsole::printf or the std::string overload for this function."
+               "\nNote that you'll need to use UTF-8 encoded strings for those functions.")]]
   void print(int x, int y, const char *fmt, ...);
+#ifndef TCOD_NO_UNICODE
   /**
    *  Print an EASCII encoded string to the console.
    *
@@ -706,6 +781,7 @@ public :
    *  .. versionadded:: 1.8
    *  \endrst
    */
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print.")]]
   void print(int x, int y, const std::string &str);
   /**
    *  Print a UTF-8 string to the console with specific alignment and blend
@@ -714,6 +790,7 @@ public :
    *  .. versionadded:: 1.8
    *  \endrst
    */
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print.")]]
   void print(int x, int y, const std::string &str,
              TCOD_alignment_t alignment, TCOD_bkgnd_flag_t flag);
   /**
@@ -725,6 +802,7 @@ public :
    *  .. versionadded:: 1.8
    *  \endrst
    */
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print and tcod::stringf.")]]
   TCODLIB_FORMAT(4, 5)
   void printf(int x, int y, const char *fmt, ...);
   /**
@@ -734,9 +812,11 @@ public :
    *  .. versionadded:: 1.8
    *  \endrst
    */
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print and tcod::stringf.")]]
   TCODLIB_FORMAT(6, 7)
   void printf(int x, int y, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment,
               const char *fmt, ...);
+#endif // TCOD_NO_UNICODE
   /**
    *  Print an EASCII formatted string to the console.
    *  \rst
@@ -745,10 +825,10 @@ public :
    *    These functions have overloads for specifying flag and alignment.
    *  \endrst
    */
-  TCOD_DEPRECATED(
+  [[deprecated(
     "Use TCODConsole::print (std::string overload) or TCODConsole::printf"
     "instead of this function."
-    "\nNote that you'll need to use UTF-8 encoded strings for those functions.")
+    "\nNote that you'll need to use UTF-8 encoded strings for those functions.")]]
   void printEx(int x, int y, TCOD_bkgnd_flag_t flag,
                TCOD_alignment_t alignment, const char *fmt, ...);
 	/**
@@ -773,6 +853,7 @@ public :
 		y <= y+h < console height
 	@Param fmt printf-like format string, eventually followed by parameters. You can use control codes to change the colors inside the string, except in C#.
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print_rect and tcod::stringf.")]]
   TCODLIB_FORMAT(6, 7)
   int printRect(int x, int y, int w, int h, const char *fmt, ...);
 
@@ -800,14 +881,15 @@ public :
 	@Param alignment defines how the strings are printed on screen.
 	@Param fmt printf-like format string, eventually followed by parameters. You can use control codes to change the colors inside the string, except in C#.
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print_rect and tcod::stringf.")]]
   TCODLIB_FORMAT(8, 9)
   int printRectEx(int x, int y, int w, int h, TCOD_bkgnd_flag_t flag,
                   TCOD_alignment_t alignment, const char *fmt, ...);
 
 	/**
 	@PageName console_print
-	@FuncTitle Compute the height of an autowrapped string
-	@FuncDesc This function returns the expected height of an autowrapped string without actually printing the string with printRect or printRectEx
+	@FuncTitle Compute the height of an auto-wrapped string
+	@FuncDesc This function returns the expected height of an auto-wrapped string without actually printing the string with printRect or printRectEx
 	@Cpp int TCODConsole::getHeightRect(int x, int y, int w, int h, const char *fmt, ...)
 
 	@C int TCOD_console_get_height_rect(TCOD_console_t con,int x, int y, int w, int h, const char *fmt, ...)
@@ -821,6 +903,7 @@ public :
 		y <= y+h < console height
 	@Param fmt printf-like format string, eventually followed by parameters. You can use control codes to change the colors inside the string, except in C#.
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::get_height_rect.")]]
   TCODLIB_FORMAT(6, 7)
   int getHeightRect(int x, int y, int w, int h, const char *fmt, ...);
 
@@ -906,25 +989,29 @@ public :
 	@Cpp void TCODConsole::print(int x, int y, const wchar_t *fmt, ...)
 	@C void TCOD_console_print_utf(TCOD_console_t con,int x, int y, const wchar_t *fmt, ...)
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print and tcod::stringf.")]]
 	void print(int x, int y, const wchar_t *fmt, ...);
 	/**
 	@PageName console_print
 	@Cpp void TCODConsole::printEx(int x, int y, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const wchar_t *fmt, ...)
 	@C void TCOD_console_print_ex_utf(TCOD_console_t con,int x, int y, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const wchar_t *fmt, ...)
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print and tcod::stringf.")]]
 	void printEx(int x, int y, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const wchar_t *fmt, ...);
 	/**
 	@PageName console_print
 	@Cpp int TCODConsole::printRect(int x, int y, int w, int h, const wchar_t *fmt, ...)
 	@C int TCOD_console_print_rect_utf(TCOD_console_t con,int x, int y, int w, int h, const wchar_t *fmt, ...)
 	*/
-   	int printRect(int x, int y, int w, int h, const wchar_t *fmt, ...);
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print_rect and tcod::stringf.")]]
+  int printRect(int x, int y, int w, int h, const wchar_t *fmt, ...);
 
 	/**
 	@PageName console_print
 	@Cpp int TCODConsole::printRectEx(int x, int y, int w, int h, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const wchar_t *fmt, ...)
 	@C int TCOD_console_print_rect_ex_utf(TCOD_console_t con,int x, int y, int w, int h, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const wchar_t *fmt, ...)
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::print_rect and tcod::stringf.")]]
 	int printRectEx(int x, int y, int w, int h, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const wchar_t *fmt, ...);
 
 	/**
@@ -932,6 +1019,7 @@ public :
 	@Cpp int TCODConsole::getHeightRect(int x, int y, int w, int h, const wchar_t *fmt, ...)
 	@C int TCOD_console_get_height_rect_utf(TCOD_console_t con,int x, int y, int w, int h, const wchar_t *fmt, ...)
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::get_height_rect.")]]
 	int getHeightRect(int x, int y, int w, int h, const wchar_t *fmt, ...);
 #endif
 
@@ -963,6 +1051,7 @@ public :
 		If false, only the background color is modified
 	@Param flag this flag defines how the cell's background color is modified. See TCOD_bkgnd_flag_t
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::draw_rect.")]]
 	void rect(int x, int y, int w, int h, bool clear, TCOD_bkgnd_flag_t flag = TCOD_BKGND_DEFAULT);
 
 	/**
@@ -985,6 +1074,7 @@ public :
 	@Param l The length of the line in cells 1 <= l <= console width - x
 	@Param flag this flag defines how the cell's background color is modified. See TCOD_bkgnd_flag_t
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::draw_rect.")]]
 	void hline(int x,int y, int l, TCOD_bkgnd_flag_t flag = TCOD_BKGND_DEFAULT);
 
 	/**
@@ -1007,6 +1097,7 @@ public :
 	@Param l The length of the line in cells 1 <= l <= console height - y
 	@Param flag this flag defines how the cell's background color is modified. See TCOD_bkgnd_flag_t
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::draw_rect.")]]
 	void vline(int x,int y, int l, TCOD_bkgnd_flag_t flag = TCOD_BKGND_DEFAULT);
 
 	/**
@@ -1039,6 +1130,7 @@ public :
 	@Param fmt if NULL, the function only draws a rectangle.
 		Else, printf-like format string, eventually followed by parameters. You can use control codes to change the colors inside the string.
 	*/
+  [[deprecated("TCODConsole methods are deprecated, this call should be replaced with tcod::draw_frame.")]]
 	TCODLIB_FORMAT(8, 9)
 	void printFrame(int x,int y,int w,int h, bool clear=true, TCOD_bkgnd_flag_t flag = TCOD_BKGND_DEFAULT, const char *fmt=NULL, ...);
 
@@ -1081,6 +1173,7 @@ public :
 	@Lua Console:getBackgroundColor()
 	@Param con in the C and Python versions, the offscreen console handler or NULL for the root console
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	TCODColor getDefaultBackground() const;
 
 	/**
@@ -1094,6 +1187,7 @@ public :
 	@Lua Console:getForegroundColor()
 	@Param con in the C and Python versions, the offscreen console handler or NULL for the root console
 	*/
+  [[deprecated("Default console parameters are deprecated.")]]
 	TCODColor getDefaultForeground() const;
 
 	/**
@@ -1110,6 +1204,7 @@ public :
 		0 <= x < console width
 		0 <= y < console height
 	*/
+  [[deprecated]]
 	TCODColor getCharBackground(int x, int y) const;
 
 	/**
@@ -1126,6 +1221,7 @@ public :
 		0 <= x < console width
 		0 <= y < console height
 	*/
+  [[deprecated]]
 	TCODColor getCharForeground(int x, int y) const;
 
 	/**
@@ -1142,6 +1238,7 @@ public :
 		0 <= x < console width
 		0 <= y < console height
 	*/
+  [[deprecated]]
 	int getChar(int x, int y) const;
 
 	/**
@@ -1179,6 +1276,7 @@ public :
 			tcod.console.flush()
 		end
 	*/
+  [[deprecated("This function does not support contexts.")]]
 	static void setFade(uint8_t fade, const TCODColor &fadingColor);
 
 	/**
@@ -1217,6 +1315,7 @@ public :
 	@C# static void TCODConsole::flush()
 	@Lua tcod.console.flush()
 	*/
+  [[deprecated("This function does not support contexts.")]]
 	static void flush();
 
 	/**
@@ -1365,167 +1464,6 @@ public :
 	*/
 	static bool isKeyPressed(TCOD_keycode_t key);
 
-	/**
-	@PageName console_key_t
-	@PageTitle 	Keyboard event structure
-	@PageFather console_input
-	@PageDesc This structure contains information about a key pressed/released by the user.
-	@C
-		typedef struct {
-			TCOD_keycode_t vk;
-			char c;
-			char text[32];
-			bool pressed;
-			bool lalt;
-			bool lctrl;
-			bool lmeta;
-			bool ralt;
-			bool rctrl;
-			bool rmeta;
-			bool shift;
-		} TCOD_key_t;
-	@Lua
-		key.KeyCode
-		key.Character
-		key.Text
-		key.Pressed
-		key.LeftAlt
-		key.LeftControl
-		key.LeftMeta
-		key.RightAlt
-		key.RightControl
-		key.RightMeta
-		key.Shift
-	@Param vk An arbitrary value representing the physical key on the keyboard. Possible values are stored in the TCOD_keycode_t enum. If no key was pressed, the value is TCODK_NONE
-	@Param c If the key correspond to a printable character, the character is stored in this field. Else, this field contains 0.
-	@Param text If vk is TCODK_TEXT, this will contain the text entered by the user.
-	@Param pressed true if the event is a key pressed, or false for a key released.
-	@Param lalt This field represents the status of the left Alt key : true => pressed, false => released.
-	@Param lctrl This field represents the status of the left Control key : true => pressed, false => released.
-	@Param lmeta This field represents the status of the left Meta (Windows/Command/..) key : true => pressed, false => released.
-	@Param ralt This field represents the status of the right Alt key : true => pressed, false => released.
-	@Param rctrl This field represents the status of the right Control key : true => pressed, false => released.
-	@Param rmeta This field represents the status of the right Meta (Windows/Command/..) key : true => pressed, false => released.
-	@Param shift This field represents the status of the shift key : true => pressed, false => released.
-	*/
-
-	/**
-	@PageName console_mouse_t
-	@PageTitle Mouse event structure
-	@PageFather console_input
-	@PageDesc This structure contains information about a mouse move/press/release event.
-	@C
-		typedef struct {
-		  int x,y;
-		  int dx,dy;
-		  int cx,cy;
-		  int dcx,dcy;
-		  bool lbutton;
-		  bool rbutton;
-		  bool mbutton;
-		  bool lbutton_pressed;
-		  bool rbutton_pressed;
-		  bool mbutton_pressed;
-		  bool wheel_up;
-		  bool wheel_down;
-		} TCOD_mouse_t;
-	@Param x,y Absolute position of the mouse cursor in pixels relative to the window top-left corner.
-	@Param dx,dy Movement of the mouse cursor since the last call in pixels.
-	@Param cx,cy Coordinates of the console cell under the mouse cursor (pixel coordinates divided by the font size).
-	@Param dcx,dcy Movement of the mouse since the last call in console cells (pixel coordinates divided by the font size).
-	@Param lbutton true if the left button is pressed.
-	@Param rbutton true if the right button is pressed.
-	@Param mbutton true if the middle button (or the wheel) is pressed.
-	@Param lbutton_pressed true if the left button was pressed and released.
-	@Param rbutton_pressed true if the right button was pressed and released.
-	@Param mbutton_pressed true if the middle button was pressed and released.
-	@Param wheel_up true if the wheel was rolled up.
-	@Param wheel_down true if the wheel was rolled down.
-	*/
-
-	/**
-	@PageName console_keycode_t
-	@PageTitle 	Key codes
-	@PageFather console_input
-	@PageDesc TCOD_keycode_t is a libtcod specific code representing a key on the keyboard.
-		For Python, replace TCODK by KEY: libtcod.KEY_NONE. C# and Lua, the value is in parenthesis. Possible values are :
-		When no key was pressed (see checkForEvent) : TCOD_NONE (NoKey)
-		Special keys :
-		TCODK_ESCAPE (Escape)
-		TCODK_BACKSPACE (Backspace)
-		TCODK_TAB (Tab)
-		TCODK_ENTER (Enter)
-		TCODK_SHIFT (Shift)
-		TCODK_CONTROL (Control)
-		TCODK_ALT (Alt)
-		TCODK_PAUSE (Pause)
-		TCODK_CAPSLOCK (CapsLock)
-		TCODK_PAGEUP (PageUp)
-		TCODK_PAGEDOWN (PageDown)
-		TCODK_END (End)
-		TCODK_HOME (Home)
-		TCODK_UP (Up)
-		TCODK_LEFT (Left)
-		TCODK_RIGHT (Right)
-		TCODK_DOWN (Down)
-		TCODK_PRINTSCREEN (Printscreen)
-		TCODK_INSERT (Insert)
-		TCODK_DELETE (Delete)
-		TCODK_LWIN (Lwin)
-		TCODK_RWIN (Rwin)
-		TCODK_APPS (Apps)
-		TCODK_KPADD (KeypadAdd)
-		TCODK_KPSUB (KeypadSubtract)
-		TCODK_KPDIV (KeypadDivide)
-		TCODK_KPMUL (KeypadMultiply)
-		TCODK_KPDEC (KeypadDecimal)
-		TCODK_KPENTER (KeypadEnter)
-		TCODK_F1 (F1)
-		TCODK_F2 (F2)
-		TCODK_F3 (F3)
-		TCODK_F4 (F4)
-		TCODK_F5 (F5)
-		TCODK_F6 (F6)
-		TCODK_F7 (F7)
-		TCODK_F8 (F8)
-		TCODK_F9 (F9)
-		TCODK_F10 (F10)
-		TCODK_F11 (F11)
-		TCODK_F12 (F12)
-		TCODK_NUMLOCK (Numlock)
-		TCODK_SCROLLLOCK (Scrolllock)
-		TCODK_SPACE (Space)
-
-		numeric keys :
-
-		TCODK_0 (Zero)
-		TCODK_1 (One)
-		TCODK_2 (Two)
-		TCODK_3 (Three)
-		TCODK_4 (Four)
-		TCODK_5 (Five)
-		TCODK_6 (Six)
-		TCODK_7 (Seven)
-		TCODK_8 (Eight)
-		TCODK_9 (Nine)
-		TCODK_KP0 (KeypadZero)
-		TCODK_KP1 (KeypadOne)
-		TCODK_KP2 (KeypadTwo)
-		TCODK_KP3 (KeypadThree)
-		TCODK_KP4 (KeypadFour)
-		TCODK_KP5 (KeypadFive)
-		TCODK_KP6 (KeypadSix)
-		TCODK_KP7 (KeypadSeven)
-		TCODK_KP8 (KeypadEight)
-		TCODK_KP9 (KeypadNine)
-
-		Any other (printable) key :
-
-		TCODK_CHAR (Char)
-
-		Codes starting with TCODK_KP represents keys on the numeric keypad (if available).
-	*/
-
 	/**
 	@PageName console_offscreen
 	@PageFather console
@@ -1650,12 +1588,14 @@ public :
 	*/
 	bool saveApf(const char *filename) const;
 
+#ifndef TCOD_NO_ZLIB
 	bool loadXp(const char *filename) {
 		return TCOD_console_load_xp(data, filename) != 0;
 	}
 	bool saveXp(const char *filename, int compress_level) {
 		return TCOD_console_save_xp(data, filename, compress_level) != 0;
 	}
+#endif // TCOD_NO_ZLIB
 
 	/**
 	@PageName console_offscreen
@@ -1748,6 +1688,7 @@ public :
 			tcod.console.flush()
 		end
 	*/
+  [[deprecated("This function has been replaced by tcod::blit.")]]
 	static void blit(const TCODConsole *src,int xSrc, int ySrc, int wSrc, int hSrc, TCODConsole *dst, int xDst, int yDst, float foreground_alpha=1.0f, float background_alpha=1.0f);
 	/**
 	@PageName console_offscreen
@@ -1789,39 +1730,110 @@ public :
 		... use off1
 		off1=nil -- release the reference
 	*/
-  TCOD_DEPRECATED("This function is a stub and will do nothing.")
+  [[deprecated("This function is a stub and will do nothing.")]]
   static void setKeyboardRepeat(int initialDelay,int interval);
-  TCOD_DEPRECATED("This function is a stub and will do nothing.")
+  [[deprecated("This function is a stub and will do nothing.")]]
   static void disableKeyboardRepeat();
+  // clang-format on
+  // Delete copy constructors.
+  TCODConsole(const TCODConsole&) = delete;
+  TCODConsole& operator=(const TCODConsole&) = delete;
+  // Supports move since version 1.19
+  TCODConsole(TCODConsole&& rhs) : data{rhs.data} { rhs.data = nullptr; }
+  TCODConsole& operator=(TCODConsole&& rhs) {
+    std::swap(data, rhs.data);
+    return *this;
+  }
 
-	virtual ~TCODConsole();
+  virtual ~TCODConsole();
 
-  TCOD_DEPRECATED("This function does nothing.")
-	void setDirty(int x, int y, int w, int h);
+  [[deprecated("This function does nothing.")]] void setDirty(int x, int y, int w, int h);
 
-
-	TCODConsole(TCOD_console_t con) : data(con) {}
-
-    // ctrl = TCOD_COLCTRL_1...TCOD_COLCTRL_5 or TCOD_COLCTRL_STOP
-	static const char *getColorControlString( TCOD_colctrl_t ctrl );
-	// ctrl = TCOD_COLCTRL_FORE_RGB or TCOD_COLCTRL_BACK_RGB
-	static const char *getRGBColorControlString( TCOD_colctrl_t ctrl, const TCODColor & col );
-  /**
-   *  Return a pointer to the underlying TCOD_Console struct.
-   *  \rst
-   *  .. versionadded:: 1.14
-   *  \endrst
+  // This conversion may be unsafe.
+  TCODConsole(TCOD_Console* console) : data{console} {}
+  /***************************************************************************
+      @brief Construct a new TCODConsole object from a tcod::ConsolePtr.
+      \rst
+      .. versionadded:: 1.19
+      \endrst
    */
-  TCOD_Console* get_data() noexcept
-  {
+  explicit TCODConsole(tcod::ConsolePtr console) : data{console.release()} {}
+
+  // ctrl = TCOD_COLCTRL_1...TCOD_COLCTRL_5 or TCOD_COLCTRL_STOP
+  static const char* getColorControlString(TCOD_colctrl_t ctrl);
+  // ctrl = TCOD_COLCTRL_FORE_RGB or TCOD_COLCTRL_BACK_RGB
+  static const char* getRGBColorControlString(TCOD_colctrl_t ctrl, const TCODColor& col);
+  /***************************************************************************
+      Return a pointer to the underlying TCOD_Console struct.
+      \rst
+      .. versionadded:: 1.14
+
+      .. versionchanged:: 1.19
+          This now returns a non-NULL pointer to the root console.
+      \endrst
+   */
+  [[nodiscard]] auto get_data() noexcept -> TCOD_Console* { return get(); }
+  [[nodiscard]] auto get_data() const noexcept -> const TCOD_Console* { return get(); }
+  /***************************************************************************
+      @brief Return a pointer to the underlying TCOD_Console struct.
+
+      @return TCOD_Console*
+      \rst
+      .. versionadded:: 1.19
+      \endrst
+   */
+  [[nodiscard]] auto get() noexcept -> TCOD_Console* {
+    if (!data) return TCOD_sys_get_internal_console();
     return data;
   }
-  const TCOD_Console* get_data() const noexcept
-  {
+  [[nodiscard]] auto get() const noexcept -> const TCOD_Console* {
+    if (!data) return TCOD_sys_get_internal_console();
     return data;
   }
-protected :
-  TCODConsole();
-  TCOD_Console* data = nullptr;
+  /***************************************************************************
+      @brief Allow implicit conversions into a TCOD_Console reference.
+
+      \rst
+      .. versionadded:: 1.19
+      \endrst
+   */
+  [[nodiscard]] operator TCOD_Console&() {
+    TCOD_Console* out = data;
+    if (!out) out = TCOD_sys_get_internal_console();
+    if (!out) throw std::logic_error("Tried to get a reference to nullptr.");
+    return *out;
+  };
+  /***************************************************************************
+      @brief Allow implicit conversions into a const TCOD_Console reference.
+
+      \rst
+      .. versionadded:: 1.19
+      \endrst
+   */
+  [[nodiscard]] operator const TCOD_Console&() const {
+    const TCOD_Console* out = data;
+    if (!out) out = TCOD_sys_get_internal_console();
+    if (!out) throw std::logic_error("Tried to get a reference to nullptr.");
+    return *out;
+  };
+  /***************************************************************************
+      @brief Allow explicit conversions into a TCOD_Console pointer.  Same as calling get_data.
+
+      \rst
+      .. versionadded:: 1.19
+      \endrst
+   */
+  [[nodiscard]] explicit operator TCOD_Console*() noexcept { return get_data(); };
+  /***************************************************************************
+      @brief Allow explicit conversions into a const TCOD_Console pointer.  Same as calling get_data.
+
+      \rst
+      .. versionadded:: 1.19
+      \endrst
+   */
+  [[nodiscard]] explicit operator const TCOD_Console*() const noexcept { return get_data(); };
+
+ protected:
+  TCOD_Console* data = nullptr;  // This should be a unique_ptr, but fixing it will break the ABI.
 };
 #endif /* _TCOD_CONSOLE_HPP */
