@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,7 +42,9 @@
 #include <wctype.h>
 #endif
 
+#ifndef TCOD_NO_UNICODE
 #include <utf8proc.h>
+#endif  // TCOD_NO_UNICODE
 
 #include "console.h"
 #include "console_drawing.h"
@@ -70,7 +72,7 @@ char* TCOD_console_vsprint(const char* fmt, va_list ap) {
   /* several static buffers in case the function is used more than once in a single function call */
   static char* msg[NB_BUFFERS] = {NULL};
   static int buflen[NB_BUFFERS] = {0};
-  static int curbuf = 0;
+  static int current_buf = 0;
   char* ret;
   bool ok = false;
   if (!msg[0]) {
@@ -85,55 +87,37 @@ char* TCOD_console_vsprint(const char* fmt, va_list ap) {
      the expected string length if the buffer is not big enough */
     va_list ap_clone;
     va_copy(ap_clone, ap);
-    int len = vsnprintf(msg[curbuf], buflen[curbuf], fmt, ap_clone);
+    int len = vsnprintf(msg[current_buf], buflen[current_buf], fmt, ap_clone);
     va_end(ap_clone);
     ok = true;
-    if (len < 0 || len >= buflen[curbuf]) {
+    if (len < 0 || len >= buflen[current_buf]) {
       /* buffer too small. */
       if (len > 0) {
-        while (buflen[curbuf] < len + 1) {
-          buflen[curbuf] *= 2;
+        while (buflen[current_buf] < len + 1) {
+          buflen[current_buf] *= 2;
         }
       } else {
-        buflen[curbuf] *= 2;
+        buflen[current_buf] *= 2;
       }
-      free(msg[curbuf]);
-      msg[curbuf] = calloc(sizeof(char), buflen[curbuf]);
+      free(msg[current_buf]);
+      msg[current_buf] = calloc(sizeof(char), buflen[current_buf]);
       ok = false;
     }
   } while (!ok);
-  ret = msg[curbuf];
-  curbuf = (curbuf + 1) % NB_BUFFERS;
+  ret = msg[current_buf];
+  current_buf = (current_buf + 1) % NB_BUFFERS;
   return ret;
 }
-/**
- *  Print a titled, framed region on a console, using default colors and
- *  alignment.
- *
- *  \param con A console pointer.
- *  \param x The starting X coordinate, the left-most position being 0.
- *  \param y The starting Y coordinate, the top-most position being 0.
- *  \param w The width of the frame.
- *  \param h The height of the frame.
- *  \param empty If true the characters inside of the frame will be cleared
- *               with spaces.
- *  \param flag The blending flag.
- *  \param fmt A format string as if passed to printf.
- *  \param ... Variadic arguments as if passed to printf.
- *
- *  This function makes assumptions about the fonts character encoding.
- *  It will fail if the font encoding is not `cp437`.
- */
 void TCOD_console_print_frame(
     TCOD_Console* con, int x, int y, int w, int h, bool empty, TCOD_bkgnd_flag_t flag, const char* fmt, ...) {
   con = TCOD_console_validate_(con);
   if (!con) {
     return;
   }
-  TCOD_console_put_char(con, x, y, TCOD_CHAR_NW, flag);
-  TCOD_console_put_char(con, x + w - 1, y, TCOD_CHAR_NE, flag);
-  TCOD_console_put_char(con, x, y + h - 1, TCOD_CHAR_SW, flag);
-  TCOD_console_put_char(con, x + w - 1, y + h - 1, TCOD_CHAR_SE, flag);
+  TCOD_console_put_char(con, x, y, 0x250C, flag);
+  TCOD_console_put_char(con, x + w - 1, y, 0x2510, flag);
+  TCOD_console_put_char(con, x, y + h - 1, 0x2514, flag);
+  TCOD_console_put_char(con, x + w - 1, y + h - 1, 0x2518, flag);
   TCOD_console_hline(con, x + 1, y, w - 2, flag);
   TCOD_console_hline(con, x + 1, y + h - 1, w - 2, flag);
   if (h > 2) {
@@ -162,15 +146,6 @@ void TCOD_console_print_frame(
     con->back = tmp;
   }
 }
-/**
- *  Print a string on a console, using default colors and alignment.
- *
- *  \param con A console pointer.
- *  \param x The starting X coordinate, the left-most position being 0.
- *  \param y The starting Y coordinate, the top-most position being 0.
- *  \param fmt A format string as if passed to printf.
- *  \param ... Variadic arguments as if passed to printf.
- */
 void TCOD_console_print(TCOD_Console* con, int x, int y, const char* fmt, ...) {
   va_list ap;
   con = TCOD_console_validate_(con);
@@ -182,17 +157,6 @@ void TCOD_console_print(TCOD_Console* con, int x, int y, const char* fmt, ...) {
       con, x, y, 0, 0, con->bkgnd_flag, con->alignment, TCOD_console_vsprint(fmt, ap), false, false);
   va_end(ap);
 }
-/**
- *  Print a string on a console, using default colors.
- *
- *  \param con A console pointer.
- *  \param x The starting X coordinate, the left-most position being 0.
- *  \param y The starting Y coordinate, the top-most position being 0.
- *  \param flag The blending flag.
- *  \param alignment The font alignment to use.
- *  \param fmt A format string as if passed to printf.
- *  \param ... Variadic arguments as if passed to printf.
- */
 void TCOD_console_print_ex(
     TCOD_Console* con, int x, int y, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const char* fmt, ...) {
   va_list ap;
@@ -200,21 +164,6 @@ void TCOD_console_print_ex(
   TCOD_console_print_internal(con, x, y, 0, 0, flag, alignment, TCOD_console_vsprint(fmt, ap), false, false);
   va_end(ap);
 }
-/**
- *  Print a string on a console constrained to a rectangle, using default
- *  colors and alignment.
- *
- *  \param con A console pointer.
- *  \param x The starting X coordinate, the left-most position being 0.
- *  \param y The starting Y coordinate, the top-most position being 0.
- *  \param w The width of the region.
- *           If 0 then the maximum width will be used.
- *  \param h The height of the region.
- *           If 0 then the maximum height will be used.
- *  \param fmt A format string as if passed to printf.
- *  \param ... Variadic arguments as if passed to printf.
- *  \return The number of lines actually printed.
- */
 int TCOD_console_print_rect(TCOD_Console* con, int x, int y, int w, int h, const char* fmt, ...) {
   int ret;
   va_list ap;
@@ -228,23 +177,6 @@ int TCOD_console_print_rect(TCOD_Console* con, int x, int y, int w, int h, const
   va_end(ap);
   return ret;
 }
-/**
- *  Print a string on a console constrained to a rectangle, using default
- *  colors.
- *
- *  \param con A console pointer.
- *  \param x The starting X coordinate, the left-most position being 0.
- *  \param y The starting Y coordinate, the top-most position being 0.
- *  \param w The width of the region.
- *           If 0 then the maximum width will be used.
- *  \param h The height of the region.
- *           If 0 then the maximum height will be used.
- *  \param flag The blending flag.
- *  \param alignment The font alignment to use.
- *  \param fmt A format string as if passed to printf.
- *  \param ... Variadic arguments as if passed to printf.
- *  \return The number of lines actually printed.
- */
 int TCOD_console_print_rect_ex(
     TCOD_Console* con,
     int x,
@@ -262,20 +194,6 @@ int TCOD_console_print_rect_ex(
   va_end(ap);
   return ret;
 }
-/**
- *  Return the number of lines that would be printed by the
- *
- *  \param con A console pointer.
- *  \param x The starting X coordinate, the left-most position being 0.
- *  \param y The starting Y coordinate, the top-most position being 0.
- *  \param w The width of the region.
- *           If 0 then the maximum width will be used.
- *  \param h The height of the region.
- *           If 0 then the maximum height will be used.
- *  \param fmt A format string as if passed to printf.
- *  \param ... Variadic arguments as if passed to printf.
- *  \return The number of lines that would have been printed.
- */
 int TCOD_console_get_height_rect(TCOD_Console* con, int x, int y, int w, int h, const char* fmt, ...) {
   int ret;
   va_list ap;
@@ -421,7 +339,7 @@ int TCOD_console_print_internal(
         }
       }
       if (split) {
-        unsigned char* oldsplit = split;
+        unsigned char* old_split = split;
         while (!isspace(*split) && split > c) {
           split--;
         }
@@ -429,7 +347,7 @@ int TCOD_console_print_internal(
           *end = '\n';
         }
         if (!isspace(*split)) {
-          split = oldsplit;
+          split = old_split;
         }
         end = split;
         bak = *split;
@@ -518,7 +436,7 @@ wchar_t* TCOD_console_vsprint_utf(const wchar_t* fmt, va_list ap) {
   /* several static buffers in case the function is used more than once in a single function call */
   static wchar_t* msg[NB_BUFFERS] = {NULL};
   static int buflen[NB_BUFFERS] = {0};
-  static int curbuf = 0;
+  static int current_buf = 0;
   wchar_t* ret;
   bool ok = false;
   if (!msg[0]) {
@@ -531,24 +449,24 @@ wchar_t* TCOD_console_vsprint_utf(const wchar_t* fmt, va_list ap) {
   do {
     /* warning ! depending on the compiler, vsnprintf return -1 or
      the expected string length if the buffer is not big enough */
-    int len = vswprintf(msg[curbuf], buflen[curbuf], fmt, ap);
+    int len = vswprintf(msg[current_buf], buflen[current_buf], fmt, ap);
     ok = true;
-    if (len < 0 || len >= buflen[curbuf]) {
+    if (len < 0 || len >= buflen[current_buf]) {
       /* buffer too small. */
       if (len > 0) {
-        while (buflen[curbuf] < len + 1) {
-          buflen[curbuf] *= 2;
+        while (buflen[current_buf] < len + 1) {
+          buflen[current_buf] *= 2;
         }
       } else {
-        buflen[curbuf] *= 2;
+        buflen[current_buf] *= 2;
       }
-      free(msg[curbuf]);
-      msg[curbuf] = calloc(sizeof(wchar_t), buflen[curbuf]);
+      free(msg[current_buf]);
+      msg[current_buf] = calloc(sizeof(wchar_t), buflen[current_buf]);
       ok = false;
     }
   } while (!ok);
-  ret = msg[curbuf];
-  curbuf = (curbuf + 1) % NB_BUFFERS;
+  ret = msg[current_buf];
+  current_buf = (current_buf + 1) % NB_BUFFERS;
   return ret;
 }
 int TCOD_console_stringLength_utf(const wchar_t* s) {
@@ -676,7 +594,7 @@ int TCOD_console_print_internal_utf(
         }
       }
       if (split) {
-        wchar_t* oldsplit = split;
+        wchar_t* old_split = split;
         while (!iswspace(*split) && split > c) {
           split--;
         }
@@ -684,7 +602,7 @@ int TCOD_console_print_internal_utf(
           *end = '\n';
         }
         if (!iswspace(*split)) {
-          split = oldsplit;
+          split = old_split;
         }
         end = split;
         bak = *split;
@@ -756,12 +674,6 @@ int TCOD_console_print_internal_utf(
   } while (c && cy < con->h && (rh == 0 || cy < y + rh));
   return cy - y + 1;
 }
-/**
- *  \rst
- *  .. deprecated:: 1.8
- *    Use :any:`TCOD_console_printf` instead.
- *  \endrst
- */
 void TCOD_console_print_utf(TCOD_Console* con, int x, int y, const wchar_t* fmt, ...) {
   va_list ap;
   con = TCOD_console_validate_(con);
@@ -773,12 +685,6 @@ void TCOD_console_print_utf(TCOD_Console* con, int x, int y, const wchar_t* fmt,
       con, x, y, 0, 0, con->bkgnd_flag, con->alignment, TCOD_console_vsprint_utf(fmt, ap), false, false);
   va_end(ap);
 }
-/**
- *  \rst
- *  .. deprecated:: 1.8
- *    Use :any:`TCOD_console_printf_ex` instead.
- *  \endrst
- */
 void TCOD_console_print_ex_utf(
     TCOD_Console* con, int x, int y, TCOD_bkgnd_flag_t flag, TCOD_alignment_t alignment, const wchar_t* fmt, ...) {
   va_list ap;
@@ -799,12 +705,6 @@ int TCOD_console_print_rect_utf(TCOD_Console* con, int x, int y, int w, int h, c
   va_end(ap);
   return ret;
 }
-/**
- *  \rst
- *  .. deprecated:: 1.8
- *    Use :any:`TCOD_console_printf_rect_ex` instead.
- *  \endrst
- */
 int TCOD_console_print_rect_ex_utf(
     TCOD_Console* con,
     int x,
@@ -822,11 +722,6 @@ int TCOD_console_print_rect_ex_utf(
   va_end(ap);
   return ret;
 }
-/**
- *  \rst
- *  .. deprecated:: 1.8
- *  \endrst
- */
 int TCOD_console_get_height_rect_utf(TCOD_Console* con, int x, int y, int w, int h, const wchar_t* fmt, ...) {
   va_list ap;
   va_start(ap, fmt);
@@ -837,6 +732,7 @@ int TCOD_console_get_height_rect_utf(TCOD_Console* con, int x, int y, int w, int
 }
 
 #endif /* NO_UNICODE */
+#ifndef TCOD_NO_UNICODE
 // ----------------------------------------------------------------------------
 // New UTF-8 parser.
 /**
@@ -862,14 +758,14 @@ static int vsprint_(char** out, const char* fmt, va_list ap) {
   vsprintf(*out, fmt, ap);
   return size;
 }
-struct FormattedPrinter {
-  const unsigned char* string;
+typedef struct FormattedPrinter {
+  const unsigned char* __restrict string;
   const unsigned char* end;
   struct TCOD_ColorRGBA fg;
   struct TCOD_ColorRGBA bg;
   const struct TCOD_ColorRGBA default_fg;
   const struct TCOD_ColorRGBA default_bg;
-};
+} FormattedPrinter;
 static TCOD_Error utf8_report_error(utf8proc_ssize_t err) {
   switch (err) {
     case UTF8PROC_ERROR_NOMEM:
@@ -892,7 +788,7 @@ static TCOD_Error utf8_report_error(utf8proc_ssize_t err) {
     An error may be returned.  `out` can be NULL.
  */
 TCOD_NODISCARD
-static TCOD_Error fp_next_raw(struct FormattedPrinter* printer, int* out) {
+static TCOD_Error fp_next_raw(FormattedPrinter* __restrict printer, int* __restrict out) {
   int codepoint;
   utf8proc_ssize_t len = utf8proc_iterate(printer->string, printer->end - printer->string, &codepoint);
   if (len < 0) {
@@ -908,7 +804,7 @@ static TCOD_Error fp_next_raw(struct FormattedPrinter* printer, int* out) {
     Output the next 3 codepoints to a TCOD_ColorRGBA struct and advance.
  */
 TCOD_NODISCARD
-static TCOD_Error fp_next_rgba(struct FormattedPrinter* printer, struct TCOD_ColorRGBA* out) {
+static TCOD_Error fp_next_rgba(FormattedPrinter* __restrict printer, struct TCOD_ColorRGBA* __restrict out) {
   int r, g, b;
   TCOD_Error err;
   if ((err = fp_next_raw(printer, &r)) < 0 || (err = fp_next_raw(printer, &g)) < 0 ||
@@ -930,7 +826,7 @@ static TCOD_Error fp_next_rgba(struct FormattedPrinter* printer, struct TCOD_Col
     then `*out` is set to a zero-width space.
  */
 TCOD_NODISCARD
-static TCOD_Error fp_next(struct FormattedPrinter* printer, int* out) {
+static TCOD_Error fp_next(FormattedPrinter* __restrict printer, int* __restrict out) {
   do {
     int codepoint;
     TCOD_Error err = fp_next_raw(printer, &codepoint);
@@ -979,8 +875,8 @@ static TCOD_Error fp_next(struct FormattedPrinter* printer, int* out) {
     Return the next non-special codepoint without advancing the string pointer.
  */
 TCOD_NODISCARD
-static TCOD_Error fp_peek(const struct FormattedPrinter* printer, int* out) {
-  struct FormattedPrinter temp = *printer;
+static TCOD_Error fp_peek(const FormattedPrinter* __restrict printer, int* __restrict out) {
+  FormattedPrinter temp = *printer;
   return fp_next(&temp, out);
 }
 /*
@@ -1023,7 +919,7 @@ TCOD_NODISCARD
 static int get_character_width(int codepoint) {
   const utf8proc_property_t* property = utf8proc_get_property(codepoint);
   if (property->category == UTF8PROC_CATEGORY_CO) {  // Private Use Area.
-    return 1;                                        // Width would otherwise be zero.
+    return 1;  // Width would otherwise be zero.
   }
   switch (property->charwidth) {
     default:
@@ -1045,13 +941,13 @@ static int get_character_width(int codepoint) {
  */
 TCOD_NODISCARD
 static TCOD_Error next_split_(
-    const struct FormattedPrinter* printer,
+    const FormattedPrinter* __restrict printer,
     int max_width,
     int can_split,
-    const unsigned char** break_point,
-    int* break_width,
-    bool* add_line_break) {
-  struct FormattedPrinter it = *printer;
+    const unsigned char** __restrict break_point,
+    int* __restrict break_width,
+    bool* __restrict add_line_break) {
+  FormattedPrinter it = *printer;
   // The break point and width of the line.
   *break_point = it.end;
   *break_width = 0;
@@ -1132,7 +1028,7 @@ static TCOD_Error next_split_(
  */
 typedef struct PrintParams {
   TCOD_Console* __restrict console;  // Can not be NULL.
-  int x;                             // Cursor starting position.
+  int x;  // Cursor starting position.
   int y;
   int width;
   int height;
@@ -1140,7 +1036,7 @@ typedef struct PrintParams {
   const TCOD_ColorRGB* __restrict rgb_bg;
   TCOD_bkgnd_flag_t flag;
   TCOD_alignment_t alignment;
-  bool can_split;   // In general `can_split = false` is deprecated.
+  bool can_split;  // In general `can_split = false` is deprecated.
   bool count_only;  // True if console is read-only.
 } PrintParams;
 TCOD_NODISCARD
@@ -1165,7 +1061,7 @@ static int printn_internal_(const PrintParams* __restrict params, size_t n, cons
   if (params->rgb_bg) {
     bg = (TCOD_ColorRGBA){params->rgb_bg->r, params->rgb_bg->g, params->rgb_bg->b, 255};
   }
-  struct FormattedPrinter printer = {
+  FormattedPrinter printer = {
       .string = (const unsigned char*)string,
       .end = (const unsigned char*)string + n,
       .fg = fg,
@@ -1288,8 +1184,8 @@ static int printn_internal_(const PrintParams* __restrict params, size_t n, cons
 }
 TCOD_NODISCARD
 static int vprintf_internal_(const PrintParams* __restrict params, const char* __restrict fmt, va_list args) {
-  char stack_buffer[512];              // This buffer attempts to prevent a heap allocation.
-  char* heap_buffer = NULL;            // Heap buffer to hold the result in case of overflow.
+  char stack_buffer[512];  // This buffer attempts to prevent a heap allocation.
+  char* heap_buffer = NULL;  // Heap buffer to hold the result in case of overflow.
   char* active_buffer = stack_buffer;  // Which buffer will be passed down.
   va_list args_copy;
   va_copy(args_copy, args);
@@ -1348,9 +1244,9 @@ TCOD_Error TCOD_console_printn(
     int x,
     int y,
     size_t n,
-    const char* str,
-    const TCOD_color_t* fg,
-    const TCOD_color_t* bg,
+    const char* __restrict str,
+    const TCOD_color_t* __restrict fg,
+    const TCOD_color_t* __restrict bg,
     TCOD_bkgnd_flag_t flag,
     TCOD_alignment_t alignment) {
   const PrintParams params = {
@@ -1376,9 +1272,9 @@ int TCOD_console_printn_rect(
     int width,
     int height,
     size_t n,
-    const char* str,
-    const TCOD_color_t* fg,
-    const TCOD_color_t* bg,
+    const char* __restrict str,
+    const TCOD_color_t* __restrict fg,
+    const TCOD_color_t* __restrict bg,
     TCOD_bkgnd_flag_t flag,
     TCOD_alignment_t alignment) {
   const PrintParams params = {
@@ -1397,7 +1293,7 @@ int TCOD_console_printn_rect(
   return printn_internal_(&params, n, str);
 }
 int TCOD_console_get_height_rect_n(
-    TCOD_Console* console, int x, int y, int width, int height, size_t n, const char* str) {
+    TCOD_Console* console, int x, int y, int width, int height, size_t n, const char* __restrict str) {
   const PrintParams params = {
       .console = TCOD_console_validate_(console),
       .x = x,
@@ -1411,7 +1307,7 @@ int TCOD_console_get_height_rect_n(
   };
   return printn_internal_(&params, n, str);
 }
-int TCOD_console_get_height_rect_wn(int width, size_t n, const char* str) {
+int TCOD_console_get_height_rect_wn(int width, size_t n, const char* __restrict str) {
   TCOD_Console console = {.w = width, .h = INT_MAX};  // get_height functions don't need a fully defined console.
   return TCOD_console_get_height_rect_n(&console, 0, 0, width, INT_MAX, n, str);
 }
@@ -1422,17 +1318,17 @@ TCOD_Error TCOD_console_printn_frame(
     int width,
     int height,
     size_t n,
-    const char* title,
-    const TCOD_color_t* fg,
-    const TCOD_color_t* bg,
+    const char* __restrict title,
+    const TCOD_color_t* __restrict fg,
+    const TCOD_color_t* __restrict bg,
     TCOD_bkgnd_flag_t flag,
-    bool empty) {
+    bool clear) {
   con = TCOD_console_validate_(con);
   if (!con) {
     TCOD_set_errorv("Console pointer must not be NULL.");
     return TCOD_E_INVALID_ARGUMENT;
   }
-  TCOD_console_draw_frame_rgb(con, x, y, width, height, NULL, fg, bg, flag, empty);
+  TCOD_console_draw_frame_rgb(con, x, y, width, height, NULL, fg, bg, flag, clear);
   if (n > 0 && title) {
     char* tmp_string = malloc(n + 2);
     if (!tmp_string) {
@@ -1450,7 +1346,7 @@ TCOD_Error TCOD_console_printn_frame(
   }
   return TCOD_E_OK;
 }
-static TCOD_Error TCOD_console_vprintf_ex(
+TCOD_Error TCOD_console_vprintf(
     TCOD_Console* __restrict console,
     int x,
     int y,
@@ -1458,7 +1354,7 @@ static TCOD_Error TCOD_console_vprintf_ex(
     const TCOD_color_t* __restrict bg,
     TCOD_bkgnd_flag_t flag,
     TCOD_alignment_t alignment,
-    const char* fmt,
+    const char* __restrict fmt,
     va_list args) {
   const PrintParams params = {
       .console = TCOD_console_validate_(console),
@@ -1479,7 +1375,7 @@ TCOD_Error TCOD_console_printf_ex(
     int y,
     TCOD_bkgnd_flag_t flag,
     TCOD_alignment_t alignment,
-    const char* fmt,
+    const char* __restrict fmt,
     ...) {
   con = TCOD_console_validate_(con);
   if (!con) {
@@ -1488,11 +1384,11 @@ TCOD_Error TCOD_console_printf_ex(
   }
   va_list args;
   va_start(args, fmt);
-  TCOD_Error err = TCOD_console_vprintf_ex(con, x, y, &con->fore, &con->back, flag, alignment, fmt, args);
+  TCOD_Error err = TCOD_console_vprintf(con, x, y, &con->fore, &con->back, flag, alignment, fmt, args);
   va_end(args);
   return err;
 }
-TCOD_Error TCOD_console_printf(TCOD_Console* __restrict con, int x, int y, const char* fmt, ...) {
+TCOD_Error TCOD_console_printf(TCOD_Console* __restrict con, int x, int y, const char* __restrict fmt, ...) {
   con = TCOD_console_validate_(con);
   if (!con) {
     TCOD_set_errorv("Console pointer must not be NULL.");
@@ -1500,12 +1396,11 @@ TCOD_Error TCOD_console_printf(TCOD_Console* __restrict con, int x, int y, const
   }
   va_list args;
   va_start(args, fmt);
-  TCOD_Error err =
-      TCOD_console_vprintf_ex(con, x, y, &con->fore, &con->back, con->bkgnd_flag, con->alignment, fmt, args);
+  TCOD_Error err = TCOD_console_vprintf(con, x, y, &con->fore, &con->back, con->bkgnd_flag, con->alignment, fmt, args);
   va_end(args);
   return err;
 }
-static int TCOD_console_vprintf_rect(
+int TCOD_console_vprintf_rect(
     struct TCOD_Console* __restrict con,
     int x,
     int y,
@@ -1532,14 +1427,14 @@ static int TCOD_console_vprintf_rect(
   return vprintf_internal_(&params, fmt, args);
 }
 int TCOD_console_printf_rect_ex(
-    struct TCOD_Console* __restrict con,
+    TCOD_Console* __restrict con,
     int x,
     int y,
     int width,
     int height,
     TCOD_bkgnd_flag_t flag,
     TCOD_alignment_t alignment,
-    const char* fmt,
+    const char* __restrict fmt,
     ...) {
   con = TCOD_console_validate_(con);
   if (!con) {
@@ -1554,7 +1449,7 @@ int TCOD_console_printf_rect_ex(
   return err;
 }
 int TCOD_console_printf_rect(
-    struct TCOD_Console* __restrict con, int x, int y, int width, int height, const char* fmt, ...) {
+    TCOD_Console* __restrict con, int x, int y, int width, int height, const char* __restrict fmt, ...) {
   con = TCOD_console_validate_(con);
   if (!con) {
     TCOD_set_errorv("Console pointer must not be NULL.");
@@ -1569,7 +1464,7 @@ int TCOD_console_printf_rect(
   return err;
 }
 int TCOD_console_get_height_rect_fmt(
-    struct TCOD_Console* __restrict con, int x, int y, int width, int height, const char* fmt, ...) {
+    TCOD_Console* __restrict con, int x, int y, int width, int height, const char* __restrict fmt, ...) {
   con = TCOD_console_validate_(con);
   if (!con) {
     TCOD_set_errorv("Console pointer must not be NULL.");
@@ -1594,14 +1489,14 @@ int TCOD_console_get_height_rect_fmt(
   return result;
 }
 TCOD_Error TCOD_console_printf_frame(
-    struct TCOD_Console* __restrict con,
+    TCOD_Console* __restrict con,
     int x,
     int y,
     int width,
     int height,
     int empty,
     TCOD_bkgnd_flag_t flag,
-    const char* fmt,
+    const char* __restrict fmt,
     ...) {
   con = TCOD_console_validate_(con);
   if (!con) {
@@ -1624,3 +1519,44 @@ TCOD_Error TCOD_console_printf_frame(
   free(str);
   return err;
 }
+int TCOD_printf_rgb(TCOD_Console* __restrict console, TCOD_PrintParamsRGB params, const char* __restrict fmt, ...) {
+  va_list args;
+  va_start(args, fmt);
+  int err = TCOD_vprintf_rgb(console, params, fmt, args);
+  va_end(args);
+  return err;
+}
+int TCOD_printn_rgb(TCOD_Console* __restrict console, TCOD_PrintParamsRGB params, int n, const char* __restrict str) {
+  PrintParams internal_params = {
+      .console = TCOD_console_validate_(console),
+      .x = params.x,
+      .y = params.y,
+      .width = params.width,
+      .height = params.height,
+      .rgb_fg = params.fg,
+      .rgb_bg = params.bg,
+      .flag = params.flag ? params.flag : TCOD_BKGND_SET,
+      .alignment = params.alignment,
+      true,
+      false};
+  int err = printn_internal_(&internal_params, n, str);
+  return err;
+}
+TCOD_PUBLIC int TCOD_vprintf_rgb(
+    TCOD_Console* __restrict console, TCOD_PrintParamsRGB params, const char* __restrict fmt, va_list args) {
+  PrintParams internal_params = {
+      .console = TCOD_console_validate_(console),
+      .x = params.x,
+      .y = params.y,
+      .width = params.width,
+      .height = params.height,
+      .rgb_fg = params.fg,
+      .rgb_bg = params.bg,
+      .flag = params.flag ? params.flag : TCOD_BKGND_SET,
+      .alignment = params.alignment,
+      true,
+      false};
+  int err = vprintf_internal_(&internal_params, fmt, args);
+  return err;
+}
+#endif  // TCOD_NO_UNICODE
