@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,8 @@
 #ifndef _TCOD_FOV_HPP
 #define _TCOD_FOV_HPP
 
+#include <utility>
+
 #include "fov.h"
 
 class TCODPath;
@@ -60,6 +62,14 @@ class TCODLIB_API TCODMap {
 		*/
 		TCODMap(int width, int height);
 
+    TCODMap(const TCODMap&) = delete;
+    TCODMap& operator=(const TCODMap&) = delete;
+    TCODMap(TCODMap&& rhs) noexcept { std::swap(data, rhs.data); };
+    TCODMap& operator=(TCODMap&& rhs) noexcept {
+      std::swap(data, rhs.data);
+      return *this;
+    };
+
 		/**
 		@PageName fov_init
 		@PageFather fov
