@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@
  */
 #ifndef TCOD_GUI_CONTAINER_HPP
 #define TCOD_GUI_CONTAINER_HPP
+#ifndef TCOD_NO_UNICODE
 #include "widget.hpp"
 class TCODLIB_GUI_API Container : public Widget {
  public:
@@ -44,6 +45,7 @@ class TCODLIB_GUI_API Container : public Widget {
   void update(const TCOD_key_t k);
 
  protected:
-  TCODList<Widget*> content;
+  TCODList<Widget*> content;  // Can't be fixed without breaking the ABI.
 };
+#endif  // TCOD_NO_UNICODE
 #endif /* TCOD_GUI_CONTAINER_HPP */
