@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "flatlist.hpp"
 
 #include <math.h>
@@ -50,21 +51,28 @@ FlatList::~FlatList() {}
 
 void FlatList::render() {
   w--;
-  boxx++;
+  box_x++;
   TextBox::render();
-  boxx--;
+  box_x--;
   w++;
-  con->setDefaultBackground((onLeftArrow) ? backFocus : back);
-  con->setDefaultForeground((onLeftArrow) ? foreFocus : fore);
-  con->putChar(x + boxx, y, TCOD_CHAR_ARROW_W);
-  con->setDefaultBackground((onRightArrow) ? backFocus : back);
-  con->setDefaultForeground((onRightArrow) ? foreFocus : fore);
-  con->putChar(x + w - 1, y, TCOD_CHAR_ARROW_E);
+  auto& console = static_cast<TCOD_Console&>(*con);
+  if (console.in_bounds({x + box_x, y})) {
+    console.at({x + box_x, y}) = {
+        0x2190,  // ←
+        TCOD_ColorRGBA(onLeftArrow ? foreFocus : fore),
+        TCOD_ColorRGBA(onLeftArrow ? backFocus : back)};
+  }
+  if (console.in_bounds({x + w - 1, y})) {
+    console.at({x + w - 1, y}) = {
+        0x2192,  // →
+        TCOD_ColorRGBA(onRightArrow ? foreFocus : fore),
+        TCOD_ColorRGBA(onRightArrow ? backFocus : back)};
+  }
 }
 
 void FlatList::update(TCOD_key_t k) {
   onLeftArrow = onRightArrow = false;
-  if (mouse.cx == x + boxx && mouse.cy == y)
+  if (mouse.cx == x + box_x && mouse.cy == y)
     onLeftArrow = true;
   else if (mouse.cx == x + w - 1 && mouse.cy == y)
     onRightArrow = true;
@@ -119,3 +127,4 @@ void FlatList::setList(const char** l) {
   value = list = l;
   valueToText();
 }
+#endif  // TCOD_NO_UNICODE
