@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,10 +31,12 @@
  */
 #ifndef TCOD_GUI_HBOX_HPP
 #define TCOD_GUI_HBOX_HPP
+#ifndef TCOD_NO_UNICODE
 #include "vbox.hpp"
 class TCODLIB_GUI_API HBox : public VBox {
  public:
   HBox(int x, int y, int padding);
   void computeSize();
 };
+#endif  // TCOD_NO_UNICODE
 #endif /* TCOD_GUI_HBOX_HPP */
