@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,15 +31,15 @@
  */
 #include "image.hpp"
 
-Image::Image(int x, int y, int w, int h, const char* tip) : Widget(x, y, w, h), back(TCODColor::black) {
+Image::Image(int x, int y, int w, int h, const char* tip) : Widget(x, y, w, h), back{0, 0, 0} {
   if (tip) setTip(tip);
 }
 
 Image::~Image() {}
 
 void Image::render() {
-  con->setDefaultBackground(back);
-  con->rect(x, y, w, h, TCOD_BKGND_SET);
+  const auto bg = TCOD_ColorRGB(back);
+  tcod::draw_rect(*con, {x, y, w, h}, ' ', std::nullopt, bg);
 }
 
 void Image::setBackgroundColor(const TCODColor col) { back = col; }
