@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,10 +29,13 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "label.hpp"
 
 #include <algorithm>
 
+#include "../console_printing.hpp"
+
 Label::Label(int x, int y, const char* label, const char* tip) : Widget(x, y, 0, 1) {
   this->label = label;
   if (tip) {
@@ -40,9 +43,9 @@ Label::Label(int x, int y, const char* label, const char* tip) : Widget(x, y, 0,
   }
 }
 void Label::render() {
-  con->setDefaultBackground(back);
-  con->setDefaultForeground(fore);
-  con->printf(x, y, TCOD_BKGND_NONE, TCOD_LEFT, "%s", label);
+  const auto fg = TCOD_ColorRGB(fore);
+  tcod::print(*con, {x, y}, label, fg, std::nullopt);
 }
 void Label::computeSize() { w = label ? static_cast<int>(strlen(label)) : 0; }
 void Label::expand(int width, int) { w = std::max(w, width); }
+#endif  // TCOD_NO_UNICODE
