@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "radiobutton.hpp"
 
 RadioButton* RadioButton::groupSelect[512];
@@ -56,7 +57,8 @@ void RadioButton::unSelectGroup(int group) { groupSelect[group] = NULL; }
 void RadioButton::render() {
   Button::render();
   if (groupSelect[group] == this) {
-    con->setChar(x, y, '>');
+    auto& console = static_cast<TCOD_Console&>(*con);
+    if (console.in_bounds({x, y})) console.at({x, y}).ch = '>';
   }
 }
 
@@ -64,3 +66,4 @@ void RadioButton::onButtonClick() {
   select();
   Button::onButtonClick();
 }
+#endif  // TCOD_NO_UNICODE
