@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,12 +29,12 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "slider.hpp"
 
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
-
 Slider::Slider(int x, int y, int w, float min, float max, const char* label, const char* tip)
     : TextBox(x, y, w, 10, label, NULL, tip),
       min(min),
@@ -67,11 +67,15 @@ void Slider::render() {
   w -= 2;
   TextBox::render();
   w += 2;
-  con->setDefaultBackground((onArrows || drag) ? backFocus : back);
-  con->setDefaultForeground((onArrows || drag) ? foreFocus : fore);
-  con->rect(x + w - 2, y, 2, 1, TCOD_BKGND_SET);
-  con->setChar(x + w - 2, y, TCOD_CHAR_ARROW_W);
-  con->setChar(x + w - 1, y, TCOD_CHAR_ARROW_E);
+  const auto fg = TCOD_ColorRGBA((onArrows || drag) ? foreFocus : fore);
+  const auto bg = TCOD_ColorRGBA((onArrows || drag) ? backFocus : back);
+  auto& console = static_cast<TCOD_Console&>(*con);
+  if (console.in_bounds({x + w - 2, y})) {
+    console.at({x + w - 2, y}) = {0x2190, fg, bg};  // ←
+  }
+  if (console.in_bounds({x + w - 1, y})) {
+    console.at({x + w - 1, y}) = {0x2192, fg, bg};  // →
+  }
 }
 
 void Slider::update(TCOD_key_t k) {
@@ -84,12 +88,12 @@ void Slider::update(TCOD_key_t k) {
     onArrows = false;
   }
   if (drag) {
-    if (dragy == -1) {
-      dragx = mouse.x;
-      dragy = mouse.y;
+    if (drag_y == -1) {
+      drag_x = mouse.x;
+      drag_y = mouse.y;
     } else {
-      float mdx = ((mouse.x - dragx) * sensitivity) / (con->getWidth() * 8);
-      float mdy = ((mouse.y - dragy) * sensitivity) / (con->getHeight() * 8);
+      float mdx = ((mouse.x - drag_x) * sensitivity) / (con->getWidth() * 8);
+      float mdy = ((mouse.y - drag_y) * sensitivity) / (con->getHeight() * 8);
       float old_value2 = value;
       if (fabs(mdy) > fabs(mdx)) {
         mdx = -mdy;
@@ -127,11 +131,10 @@ void Slider::setValue(float value_) {
   this->value = CLAMP(min, max, value_);
   valueToText();
 }
-
 void Slider::onButtonPress() {
   if (onArrows) {
     drag = true;
-    dragy = -1;
+    drag_y = -1;
     dragValue = value;
     TCODMouse::showCursor(false);
   }
@@ -144,3 +147,4 @@ void Slider::onButtonRelease() {
     TCODMouse::showCursor(true);
   }
 }
+#endif  // TCOD_NO_UNICODE
