@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@
  */
 #ifndef TCOD_GUI_SLIDER_HPP
 #define TCOD_GUI_SLIDER_HPP
+#ifndef TCOD_NO_UNICODE
 #include "textbox.hpp"
 class TCODLIB_GUI_API Slider : public TextBox {
  public:
@@ -54,8 +55,8 @@ class TCODLIB_GUI_API Slider : public TextBox {
   float min, max, value, sensitivity;
   bool onArrows;
   bool drag;
-  int dragx;
-  int dragy;
+  int drag_x;
+  int drag_y;
   float dragValue;
   char* fmt;
   void (*cbk)(Widget* wid, float val, void* data);
@@ -66,4 +67,5 @@ class TCODLIB_GUI_API Slider : public TextBox {
   void onButtonPress();
   void onButtonRelease();
 };
+#endif  // TCOD_NO_UNICODE
 #endif /* TCOD_GUI_SLIDER_HPP */
