@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,13 +29,17 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "statusbar.hpp"
 
+#include "../console_printing.hpp"
+
 void StatusBar::render() {
-  con->setDefaultBackground(back);
-  con->rect(x, y, w, h, true, TCOD_BKGND_SET);
+  const auto bg = TCOD_ColorRGB(back);
+  tcod::draw_rect(*con, {x, y, w, h}, ' ', std::nullopt, bg);
   if (focus && focus->tip) {
-    con->setDefaultForeground(fore);
-    con->printRectEx(x + 1, y, w, h, TCOD_BKGND_NONE, TCOD_LEFT, "%s", focus->tip);
+    const auto fg = TCOD_ColorRGB(fore);
+    tcod::print_rect(*con, {x + 1, y, w, h}, focus->tip, fg, std::nullopt);
   }
 }
+#endif  // TCOD_NO_UNICODE
