@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,10 +31,12 @@
  */
 #ifndef TCOD_GUI_STATUSBAR_HPP
 #define TCOD_GUI_STATUSBAR_HPP
+#ifndef TCOD_NO_UNICODE
 #include "widget.hpp"
 class TCODLIB_GUI_API StatusBar : public Widget {
  public:
   StatusBar(int x, int y, int w, int h) : Widget(x, y, w, h) {}
   void render();
 };
+#endif  // TCOD_NO_UNICODE
 #endif /* TCOD_GUI_STATUSBAR_HPP */
