@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,35 +29,41 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "textbox.hpp"
 
 #include <stdio.h>
 #include <string.h>
 
+#include <algorithm>
+#include <utility>
+
+#include "../console_printing.hpp"
+
 float TextBox::blinkingDelay = 0.5f;
-TextBox::TextBox(int x, int y, int w, int maxw, const char* label, const char* value, const char* tip)
+TextBox::TextBox(int x, int y, int w, int max_width, const char* label, const char* value, const char* tip)
     : Widget(x, y, w, 1),
       txt(NULL),
       blink(0.0f),
       pos(0),
       offset(0),
-      maxw(maxw),
+      max_width(max_width),
       insert(true),
-      txtcbk(NULL),
+      text_callback(NULL),
       data(NULL) {
-  if (maxw > 0) {
-    txt = new char[maxw + 1];
-    memset(txt, 0, sizeof(char) * (maxw + 1));
+  if (max_width > 0) {
+    txt = new char[max_width + 1];
+    memset(txt, 0, sizeof(char) * (max_width + 1));
     if (value) {
-      strncpy(txt, value, maxw);
+      strncpy(txt, value, max_width);
     }
   }
   if (tip) setTip(tip);
   if (label) this->label = TCOD_strdup(label);
-  boxw = w;
+  box_width = w;
   if (label) {
-    boxx = static_cast<int>(strlen(label) + 1);
-    this->w += boxx;
+    box_x = static_cast<int>(strlen(label) + 1);
+    this->w += box_x;
   }
 }
 
@@ -66,46 +72,44 @@ TextBox::~TextBox() {
   if (label) free(label);
 }
 
-void TextBox::setText(const char* txt_) { strncpy(this->txt, txt_, maxw); }
+void TextBox::setText(const char* txt_) { strncpy(this->txt, txt_, max_width); }
 
 void TextBox::render() {
-  con->setDefaultBackground(back);
-  con->setDefaultForeground(fore);
-  con->rect(x, y, w, h, true, TCOD_BKGND_SET);
-  if (label) con->printf(x, y, TCOD_BKGND_NONE, TCOD_LEFT, "%s", label);
+  TCOD_Console& console = *con;
+  const auto bg = TCOD_ColorRGB(back);
+  tcod::draw_rect(console, {x, y, w, h}, ' ', std::nullopt, bg);
+  const auto fg = TCOD_ColorRGB(fore);
+  if (label) tcod::print(console, {x, y}, label, fg, std::nullopt);
 
-  con->setDefaultBackground(keyboardFocus == this ? foreFocus : fore);
-  con->setDefaultForeground(keyboardFocus == this ? backFocus : back);
-  con->rect(x + boxx, y, boxw, h, false, TCOD_BKGND_SET);
-  int len = static_cast<int>(strlen(txt) - offset);
-  if (len > boxw) len = boxw;
-  if (txt) con->printf(x + boxx, y, TCOD_BKGND_NONE, TCOD_LEFT, "%.*s", len, &txt[offset]);
-  if (keyboardFocus == this && blink > 0.0f) {
-    if (insert) {
-      con->setCharBackground(x + boxx + pos - offset, y, fore);
-      con->setCharForeground(x + boxx + pos - offset, y, back);
-    } else {
-      con->setCharBackground(x + boxx + pos - offset, y, back);
-      con->setCharForeground(x + boxx + pos - offset, y, fore);
-    }
+  const auto focus_bg = TCOD_ColorRGB(keyboardFocus == this ? backFocus : back);
+  tcod::draw_rect(console, {x + box_x, y, box_width, h}, 0, std::nullopt, focus_bg);
+  const int len = std::min(static_cast<int>(strlen(txt) - offset), box_width);
+  const auto focus_fg = TCOD_ColorRGB(keyboardFocus == this ? foreFocus : fore);
+  if (txt) tcod::print(console, {x + box_x, y}, tcod::stringf("%.*s", len, &txt[offset]), focus_fg, std::nullopt);
+
+  if (keyboardFocus == this && blink > 0.0f && console.in_bounds({x + box_x + pos - offset, y})) {
+    auto& tile = console.at(x + box_x + pos - offset, y);
+    tile.fg = TCOD_ColorRGBA(fore);
+    tile.bg = TCOD_ColorRGBA(back);
+    if (insert) std::swap(tile.fg, tile.bg);
   }
 }
 
 void TextBox::update(TCOD_key_t k) {
-  if (keyboardFocus == this) {
+  if (keyboardFocus == this && k.pressed) {
     blink -= elapsed;
     if (blink < -blinkingDelay) blink += 2 * blinkingDelay;
     if (k.vk == TCODK_CHAR || (k.vk >= TCODK_0 && k.vk <= TCODK_9) || (k.vk >= TCODK_KP0 && k.vk <= TCODK_KP9)) {
-      if (!insert || static_cast<int>(strlen(txt)) < maxw) {
+      if (!insert || static_cast<int>(strlen(txt)) < max_width) {
         if (insert && pos < static_cast<int>(strlen(txt))) {
           for (int i = static_cast<int>(strlen(txt)); i >= pos; --i) {
             txt[i + 1] = txt[i];
           }
         }
         txt[pos] = k.c;
-        if (pos < maxw) pos++;
+        if (pos < max_width) pos++;
         if (pos >= w) offset = pos - w + 1;
-        if (txtcbk) txtcbk(this, txt, data);
+        if (text_callback) text_callback(this, txt, data);
       }
       blink = blinkingDelay;
     }
@@ -132,7 +136,7 @@ void TextBox::update(TCOD_key_t k) {
           for (uint32_t i = pos; i <= strlen(txt); i++) {
             txt[i] = txt[i + 1];
           }
-          if (txtcbk) txtcbk(this, txt, data);
+          if (text_callback) text_callback(this, txt, data);
           if (pos < offset) offset = pos;
         }
         blink = blinkingDelay;
@@ -142,7 +146,7 @@ void TextBox::update(TCOD_key_t k) {
           for (uint32_t i = pos; i <= strlen(txt); i++) {
             txt[i] = txt[i + 1];
           }
-          if (txtcbk) txtcbk(this, txt, data);
+          if (text_callback) text_callback(this, txt, data);
         }
         blink = blinkingDelay;
         break;
@@ -165,5 +169,6 @@ void TextBox::update(TCOD_key_t k) {
 }
 
 void TextBox::onButtonClick() {
-  if (mouse.cx >= x + boxx && mouse.cx < x + boxx + boxw) keyboardFocus = this;
+  if (mouse.cx >= x + box_x && mouse.cx < x + box_x + box_width) keyboardFocus = this;
 }
+#endif  // TCOD_NO_UNICODE
