@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,17 +31,18 @@
  */
 #ifndef TCOD_GUI_TEXTBOX_HPP
 #define TCOD_GUI_TEXTBOX_HPP
+#ifndef TCOD_NO_UNICODE
 #include "widget.hpp"
 class TCODLIB_GUI_API TextBox : public Widget {
  public:
-  TextBox(int x, int y, int w, int maxw, const char* label, const char* value, const char* tip = NULL);
+  TextBox(int x, int y, int w, int max_width, const char* label, const char* value, const char* tip = NULL);
   virtual ~TextBox();
   void render();
   void update(const TCOD_key_t k);
   void setText(const char* txt);
   const char* getValue() { return txt; }
   void setCallback(void (*cbk)(Widget* wid, char* val, void* data), void* data_) {
-    txtcbk = cbk;
+    text_callback = cbk;
     this->data = data_;
   }
   static void setBlinkingDelay(float delay) { blinkingDelay = delay; }
@@ -52,11 +53,12 @@ class TCODLIB_GUI_API TextBox : public Widget {
   char* txt;
   float blink;
   int pos, offset;
-  int boxx, boxw, maxw;
+  int box_x, box_width, max_width;
   bool insert;
-  void (*txtcbk)(Widget* wid, char* val, void* data);
+  void (*text_callback)(Widget* wid, char* val, void* data);
   void* data;
 
   void onButtonClick();
 };
+#endif  // TCOD_NO_UNICODE
 #endif /* TCOD_GUI_TEXTBOX_HPP */
