@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,20 +29,20 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "togglebutton.hpp"
 
 #include <string.h>
 
+#include "../console_printing.hpp"
+
 void ToggleButton::render() {
-  con->setDefaultBackground(mouseIn ? backFocus : back);
-  con->setDefaultForeground(mouseIn ? foreFocus : fore);
-  con->rect(x, y, w, h, true, TCOD_BKGND_SET);
+  const auto fg = TCOD_ColorRGB(mouseIn ? foreFocus : fore);
+  const auto bg = TCOD_ColorRGB(mouseIn ? backFocus : back);
+  tcod::draw_rect(*con, {x, y, w, h}, ' ', std::nullopt, bg);
   const char* check = pressed ? u8"\u2611" : u8"\u2610";
-  if (label) {
-    con->printf(x, y, TCOD_BKGND_NONE, TCOD_LEFT, "%s %s", check, label);
-  } else {
-    con->printf(x, y, TCOD_BKGND_NONE, TCOD_LEFT, "%s", check);
-  }
+  const auto text = label ? tcod::stringf("%s %s", check, label) : check;
+  tcod::print(*con, {x, y}, text, fg, std::nullopt);
 }
 
 void ToggleButton::onButtonPress() {}
@@ -53,3 +53,4 @@ void ToggleButton::onButtonClick() {
   pressed = !pressed;
   if (cbk) cbk(this, userData);
 }
+#endif  // TCOD_NO_UNICODE
