@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,12 +29,15 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+#ifndef TCOD_NO_UNICODE
 #include "toolbar.hpp"
 
 #include <string.h>
 
 #include <algorithm>
 
+#include "../console_printing.hpp"
+
 class Separator : public Widget {
  public:
   Separator(const char* txt, const char* tip = NULL) : Widget(0, 0, 0, 1), txt(NULL) {
@@ -51,14 +54,17 @@ class Separator : public Widget {
   void computeSize() { w = txt ? static_cast<int>(strlen(txt) + 2) : 0; }
   void expand(int width, int) { w = std::max(w, width); }
   void render() {
-    con->setDefaultBackground(back);
-    con->setDefaultForeground(fore);
-    con->hline(x, y, w, TCOD_BKGND_SET);
-    con->setChar(x - 1, y, TCOD_CHAR_TEEE);
-    con->setChar(x + w, y, TCOD_CHAR_TEEW);
-    con->setDefaultBackground(fore);
-    con->setDefaultForeground(back);
-    con->printf(x + w / 2, y, TCOD_BKGND_SET, TCOD_CENTER, " %s ", txt);
+    auto& console = static_cast<TCOD_Console&>(*con);
+    const auto fg = TCOD_ColorRGB(fore);
+    const auto bg = TCOD_ColorRGB(back);
+    tcod::draw_rect(console, {x, y, w, 1}, 0x2500, fg, bg);
+    if (console.in_bounds({x - 1, y})) {
+      console.at({x - 1, y}) = {0x251C, {fg.r, fg.g, fg.b, 255}, {bg.r, bg.g, bg.b, 255}};  // ├
+    }
+    if (console.in_bounds({x + w, y})) {
+      console.at({x + w, y}) = {0x2524, {fg.r, fg.g, fg.b, 255}, {bg.r, bg.g, bg.b, 255}};  // ┤
+    }
+    tcod::print(console, {x + w / 2, y}, tcod::stringf(" %s ", txt), bg, fg, TCOD_CENTER);
   }
   char* txt;
 };
@@ -124,3 +130,4 @@ void ToolBar::computeSize() {
   }
 }
 void ToolBar::addSeparator(const char* txt, const char* tip_) { addWidget(new Separator(txt, tip_)); }
+#endif  // TCOD_NO_UNICODE
