@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -78,7 +78,11 @@ void Widget::setForegroundColor(const TCODColor col, const TCODColor colFocus) {
 void Widget::setConsole(TCODConsole* console) { con = console; }
 
 void Widget::update(const TCOD_key_t) {
-  bool curs = TCODMouse::isCursorVisible();
+#ifndef NO_SDL
+  const bool curs = TCODMouse::isCursorVisible();
+#else
+  const bool curs = false;
+#endif  // NO_SDL
   if (curs) {
     if (mouse.cx >= x && mouse.cx < x + w && mouse.cy >= y && mouse.cy < y + h) {
       if (!mouseIn) {
@@ -125,8 +129,8 @@ void Widget::updateWidgetsIntern(const TCOD_key_t k) {
   }
 }
 
-void Widget::updateWidgets(const TCOD_key_t k, const TCOD_mouse_t pmouse) {
-  mouse = pmouse;
+void Widget::updateWidgets(const TCOD_key_t k, const TCOD_mouse_t p_mouse) {
+  mouse = p_mouse;
   updateWidgetsIntern(k);
 }
 
