@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,9 @@
 
 #include <stdexcept>
 
+#ifndef NO_SDL
 TCODImage::TCODImage(const char* filename) : deleteData(true) { data = TCOD_image_load(filename); }
+#endif  // NO_SDL
 
 TCODImage::TCODImage(int width, int height) : deleteData(true) { data = TCOD_image_new(width, height); }
 
@@ -50,7 +52,7 @@ void TCODImage::clear(const TCODColor col) {
 void TCODImage::getSize(int* w, int* h) const { TCOD_image_get_size(data, w, h); }
 
 TCODImage::~TCODImage() {
-  if (deleteData) TCOD_image_delete(data);
+  if (deleteData && data) TCOD_image_delete(data);
 }
 
 TCODColor TCODImage::getPixel(int x, int y) const { return TCOD_image_get_pixel(data, x, y); }
@@ -76,11 +78,13 @@ void TCODImage::blitRect(TCODConsole* console, int x, int y, int w, int h, TCOD_
   TCOD_image_blit_rect(data, console->get_data(), x, y, w, h, bkgnd_flag);
 }
 
+#ifndef NO_SDL
 void TCODImage::save(const char* filename) const {
   if (TCOD_image_save(data, filename) < 0) {
     throw std::runtime_error(TCOD_get_error());
   }
 }
+#endif  // NO_SDL
 
 void TCODImage::setKeyColor(const TCODColor keyColor) {
   TCOD_color_t color = {keyColor.r, keyColor.g, keyColor.b};
