@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -117,6 +117,6 @@ TCODLIB_API bool TCOD_image_is_pixel_transparent(const TCOD_Image* image, int x,
 
 #ifdef __cplusplus
 }
-#endif
-
+namespace tcod {}  // namespace tcod
+#endif  // __cplusplus
 #endif /* _TCOD_IMAGE_H */
