@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,21 @@
 
 #include "console.hpp"
 #include "image.h"
+#include "matrix.hpp"
 
+namespace tcod {
+struct ImageDeleter {
+  void operator()(TCOD_Image* image) const { TCOD_image_delete(image); }
+};
+/***************************************************************************
+    @brief A unique pointer to a TCOD_Image.
+
+    \rst
+    .. versionadded:: 1.24
+    \endrst
+ */
+typedef std::unique_ptr<TCOD_Image, ImageDeleter> ImagePtr;
+}  // namespace tcod
 class TCODConsole;
 
 class TCODLIB_API TCODImage {
@@ -49,6 +63,9 @@ public :
 	@PageDesc This toolkit contains some image manipulation utilities.
 	*/
 
+  /// @brief Default constructs an image.  This will be in a partially invalid state until assigned a real image.
+  TCODImage() noexcept = default;
+
 	/**
 	@PageName image_create
 	@PageTitle Creating an image
@@ -99,6 +116,45 @@ public :
 	*/
 	TCODImage(const TCODConsole *console);
 
+  /***************************************************************************
+      @brief Take ownership of an image pointer.
+
+      \rst
+      .. versionadded:: 1.24
+      \endrst
+   */
+  TCODImage(tcod::ImagePtr image) noexcept: data{image.release()}, deleteData{true} {};
+
+  TCODImage(const TCODImage&) = delete;
+  TCODImage& operator=(const TCODImage&) = delete;
+  TCODImage(TCODImage&& rhs) noexcept {
+    std::swap(data, rhs.data);
+    std::swap(deleteData, rhs.deleteData);
+  };
+  TCODImage& operator=(TCODImage&& rhs) noexcept {
+    std::swap(data, rhs.data);
+    std::swap(deleteData, rhs.deleteData);
+    return *this;
+  };
+
+  // clang-format on
+  /***************************************************************************
+      @brief Construct a new TCODImage object from a Matrix of pixels.
+
+      This constructor is provisional.
+
+      @param pixels A 2D matrix of RGB pixels.
+   */
+  explicit TCODImage(const tcod::Matrix<TCOD_ColorRGB, 2>& pixels)
+      : TCODImage(pixels.get_shape().at(0), pixels.get_shape().at(1)) {
+    for (int y = 0; y < pixels.get_shape().at(1); ++y) {
+      for (int x = 0; x < pixels.get_shape().at(0); ++x) {
+        putPixel(x, y, pixels[{x, y}]);
+      }
+    }
+  }
+  // clang-format off
+
 	/**
 	@PageName image_create
 	@FuncTitle refreshing an image created from a console
@@ -151,6 +207,18 @@ public :
 	*/
 	void getSize(int *w,int *h) const;
 
+  /***************************************************************************
+      @brief Get the {width, height} of this image.
+
+      \rst
+      .. versionadded:: 1.24
+      \endrst
+   */
+  [[nodiscard]] auto getSize() const noexcept -> std::array<int, 2> {
+    std::array<int, 2> out{};
+    TCOD_image_get_size(data, &out[0], &out[1]);
+    return out;
+  }
 	/**
 	@PageName image_read
 	@FuncTitle Getting the color of a pixel
@@ -367,6 +435,9 @@ public :
 	@Param flag	This flag defines how the cell's background color is modified. See TCOD_bkgnd_flag_t.
 	*/
 	void blitRect(TCODConsole *console, int x, int y, int w=-1, int h=-1, TCOD_bkgnd_flag_t bkgnd_flag = TCOD_BKGND_SET ) const;
+	void blitRect(TCOD_Console &console, int x, int y, int w=-1, int h=-1, TCOD_bkgnd_flag_t bkgnd_flag = TCOD_BKGND_SET ) const {
+    TCOD_image_blit_rect(data, &console, x, y, w, h, bkgnd_flag);
+  }
 
 	/**
 	@PageName image_blit
@@ -390,6 +461,9 @@ public :
 	@Param angle	Rotation angle in radians.
 	*/
 	void blit(TCODConsole *console, float x, float y, TCOD_bkgnd_flag_t bkgnd_flag = TCOD_BKGND_SET, float scale_x=1.0f, float scale_y=1.0f, float angle=0.0f) const;
+	void blit(TCOD_Console& console, float x, float y, TCOD_bkgnd_flag_t bkgnd_flag = TCOD_BKGND_SET, float scale_x=1.0f, float scale_y=1.0f, float angle=0.0f) const {
+    TCOD_image_blit(data, &console, x, y, bkgnd_flag, scale_x, scale_y, angle);
+  }
 
 	/**
 	@PageName image_blit
@@ -443,6 +517,10 @@ public :
 	@Param sx,sy,w,h	Part of the image to blit. Use -1 in w and h to blit the whole image.
 	*/
 	void blit2x(TCODConsole *dest, int dx, int dy, int sx=0, int sy=0, int w=-1, int h=-1) const;
+  [[deprecated("This call is replaced by tcod::draw_quartergraphics.")]]
+	void blit2x(TCOD_Console& dest, int dx, int dy, int sx=0, int sy=0, int w=-1, int h=-1) const {
+    TCOD_image_blit_2x(data, &dest, dx, dy, sx, sy, w, h);
+  }
 
   /**
       Return the pointer to this objects TCOD_Image data.
@@ -461,13 +539,58 @@ public :
    */
   const TCOD_Image* get_data() const noexcept { return data; }
 
-	TCODImage(TCOD_image_t img) : data(img), deleteData(false) {}
+	[[deprecated("This only makes a reference to the image data."
+               "  If you intended to pass ownership then use `TCODImage{tcod::ImagePtr{image_ptr}}`")]]
+  TCODImage(TCOD_image_t img) : data(img), deleteData(false) {}
 	virtual ~TCODImage();
+  /***************************************************************************
+      @brief Allow implicit conversions to TCOD_Image&.
+      \rst
+      .. versionadded:: 1.19
+      \endrst
+   */
+  [[nodiscard]] operator TCOD_Image&() { return *data; }
+  /***************************************************************************
+      @brief Allow implicit conversions to const TCOD_Image&.
+      \rst
+      .. versionadded:: 1.19
+      \endrst
+   */
+  [[nodiscard]] operator const TCOD_Image&() const { return *data; }
 
 protected :
 	friend class TCODLIB_API TCODSystem;
 	friend class TCODLIB_API TCODZip;
-	struct TCOD_Image *data;
-	bool deleteData;
+	struct TCOD_Image *data{nullptr};
+	bool deleteData{false};
 };
+// clang-format on
+namespace tcod {
+/***************************************************************************
+    @brief Draw a double resolution image on a console using quadrant character glyphs.
+
+    @param dest The console to draw to.
+    @param source The source image which will be rendered.
+    @param dest_xy The upper-left position to where the source will be drawn.
+    @param source_rect The `{left, top, width, height}` region of the source image to draw.
+        A width or height of -1 will use the full size of the image.
+
+    @code{.cpp}
+      auto console = tcod::Console{80, 50};
+      TCODImage* image = new TCODImage(console.get_width() * 2, console.get_height() * 2);
+      tcod::draw_quartergraphics(console, image);
+    @endcode
+    \rst
+    .. versionadded:: 1.19
+    \endrst
+ */
+inline void draw_quartergraphics(
+    TCOD_Console& dest,
+    const TCOD_Image& source,
+    const std::array<int, 2>& dest_xy = {0, 0},
+    const std::array<int, 4>& src_rect = {0, 0, -1, -1}) {
+  TCOD_image_blit_2x(
+      &source, &dest, dest_xy.at(0), dest_xy.at(1), src_rect.at(0), src_rect.at(1), src_rect.at(2), src_rect.at(3));
+}
+}  // namespace tcod
 #endif /* _TCOD_IMAGE_HPP */
