@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,9 +56,7 @@ static void TCOD_image_invalidate_mipmaps(TCOD_Image* image) {
     Return true if `x` and `y` are in the bounds of `image`.
  */
 static bool TCOD_image_in_bounds(const TCOD_Image* image, int x, int y) {
-  if (!image) {
-    return 0;
-  }
+  if (!image) return false;
   return (0 <= x && 0 <= y && x < image->mipmaps[0].width && y < image->mipmaps[0].height);
 }
 
@@ -153,11 +151,11 @@ TCOD_Image* TCOD_image_new(int width, int height) {
   return ret;
 }
 void TCOD_image_get_size(const TCOD_Image* image, int* w, int* h) {
-  if (!image) {
-    return;
-  }
-  *w = image->mipmaps[0].width;
-  *h = image->mipmaps[0].height;
+  if (w) *w = 0;
+  if (h) *h = 0;
+  if (!image) return;
+  if (w) *w = image->mipmaps[0].width;
+  if (h) *h = image->mipmaps[0].height;
 }
 
 TCOD_color_t TCOD_image_get_pixel(const TCOD_Image* image, int x, int y) {
@@ -572,18 +570,18 @@ void TCOD_image_rotate90(TCOD_Image* image, int numRotations) {
     }
   } else if (numRotations == 3) {
     /* rotate 270 degrees */
-    TCOD_Image* newImg = TCOD_image_new(height, width);
+    TCOD_Image* new_image = TCOD_image_new(height, width);
     for (int px = 0; px < width; ++px) {
       for (int py = 0; py < height; ++py) {
         TCOD_color_t col1 = TCOD_image_get_pixel(image, px, py);
-        TCOD_image_put_pixel(newImg, py, width - 1 - px, col1);
+        TCOD_image_put_pixel(new_image, py, width - 1 - px, col1);
       }
     }
     TCOD_image_delete_internal(image);
     /* update image with the new image content */
-    image->mipmaps = newImg->mipmaps;
-    image->nb_mipmaps = newImg->nb_mipmaps;
-    free(newImg);
+    image->mipmaps = new_image->mipmaps;
+    image->nb_mipmaps = new_image->nb_mipmaps;
+    free(new_image);
   }
 }
 
@@ -599,7 +597,7 @@ void TCOD_image_scale(TCOD_Image* image, int new_w, int new_h) {
   if (new_w == 0 || new_h == 0) {
     return;
   }
-  TCOD_Image* newimg = TCOD_image_new(new_w, new_h);
+  TCOD_Image* new_image = TCOD_image_new(new_w, new_h);
 
   if (new_w < width && new_h < height) {
     /* scale down image, using supersampling */
@@ -625,7 +623,7 @@ void TCOD_image_scale(TCOD_Image* image, int new_w, int new_h) {
         float x1weight = (x1 - x1floor);
         int ix1 = (int)x1floor;
 
-        float r = 0, g = 0, b = 0, sumweight = 0.0f;
+        float r = 0, g = 0, b = 0, sum_weight = 0.0f;
         /* left & right fractional edges */
         for (int srcy = (int)y0 + 1; srcy < (int)y1; ++srcy) {
           TCOD_color_t col_left = TCOD_image_get_pixel(image, ix0, srcy);
@@ -633,7 +631,7 @@ void TCOD_image_scale(TCOD_Image* image, int new_w, int new_h) {
           r += col_left.r * x0weight + col_right.r * x1weight;
           g += col_left.g * x0weight + col_right.g * x1weight;
           b += col_left.b * x0weight + col_right.b * x1weight;
-          sumweight += x0weight + x1weight;
+          sum_weight += x0weight + x1weight;
         }
         /* top & bottom fractional edges */
         for (int srcx = (int)x0 + 1; srcx < (int)x1; ++srcx) {
@@ -642,7 +640,7 @@ void TCOD_image_scale(TCOD_Image* image, int new_w, int new_h) {
           r += col_top.r * y0weight + col_bottom.r * y1weight;
           g += col_top.g * y0weight + col_bottom.g * y1weight;
           b += col_top.b * y0weight + col_bottom.b * y1weight;
-          sumweight += y0weight + y1weight;
+          sum_weight += y0weight + y1weight;
         }
         /* center */
         for (int srcy = (int)y0 + 1; srcy < (int)y1; ++srcy) {
@@ -651,7 +649,7 @@ void TCOD_image_scale(TCOD_Image* image, int new_w, int new_h) {
             r += sample.r;
             g += sample.g;
             b += sample.b;
-            sumweight += 1.0f;
+            sum_weight += 1.0f;
           }
         }
         /* corners */
@@ -659,30 +657,30 @@ void TCOD_image_scale(TCOD_Image* image, int new_w, int new_h) {
         r += col.r * (x0weight * y0weight);
         g += col.g * (x0weight * y0weight);
         b += col.b * (x0weight * y0weight);
-        sumweight += x0weight * y0weight;
+        sum_weight += x0weight * y0weight;
         col = TCOD_image_get_pixel(image, ix0, iy1);
         r += col.r * (x0weight * y1weight);
         g += col.g * (x0weight * y1weight);
         b += col.b * (x0weight * y1weight);
-        sumweight += x0weight * y1weight;
+        sum_weight += x0weight * y1weight;
         col = TCOD_image_get_pixel(image, ix1, iy1);
         r += col.r * (x1weight * y1weight);
         g += col.g * (x1weight * y1weight);
         b += col.b * (x1weight * y1weight);
-        sumweight += x1weight * y1weight;
+        sum_weight += x1weight * y1weight;
         col = TCOD_image_get_pixel(image, ix1, iy0);
         r += col.r * (x1weight * y0weight);
         g += col.g * (x1weight * y0weight);
         b += col.b * (x1weight * y0weight);
-        sumweight += x1weight * y0weight;
-        sumweight = 1.0f / sumweight;
-        r = r * sumweight + 0.5f;
-        g = g * sumweight + 0.5f;
-        b = b * sumweight + 0.5f;
+        sum_weight += x1weight * y0weight;
+        sum_weight = 1.0f / sum_weight;
+        r = r * sum_weight + 0.5f;
+        g = g * sum_weight + 0.5f;
+        b = b * sum_weight + 0.5f;
         col.r = (uint8_t)r;
         col.g = (uint8_t)g;
         col.b = (uint8_t)b;
-        TCOD_image_put_pixel(newimg, px, py, col);
+        TCOD_image_put_pixel(new_image, px, py, col);
       }
     }
   } else {
@@ -691,205 +689,196 @@ void TCOD_image_scale(TCOD_Image* image, int new_w, int new_h) {
       int srcy = py * height / new_h;
       for (int px = 0; px < new_w; ++px) {
         int srcx = px * width / new_w;
-        TCOD_image_put_pixel(newimg, px, py, TCOD_image_get_pixel(image, srcx, srcy));
+        TCOD_image_put_pixel(new_image, px, py, TCOD_image_get_pixel(image, srcx, srcy));
       }
     }
   }
   /* destroy old image */
   TCOD_image_delete_internal(image);
   /* update image with the new image content */
-  image->mipmaps = newimg->mipmaps;
-  image->nb_mipmaps = newimg->nb_mipmaps;
-  free(newimg);
+  image->mipmaps = new_image->mipmaps;
+  image->nb_mipmaps = new_image->nb_mipmaps;
+  free(new_image);
 }
 
-/* distance between two colors */
-int rgbdist(const TCOD_color_t* c1, const TCOD_color_t* c2) {
-  int dr = (int)c1->r - c2->r;
-  int dg = (int)c1->g - c2->g;
-  int db = (int)c1->b - c2->b;
+// Return the squared distance between two colors.
+static int rgb_squared_distance(const TCOD_ColorRGB* c1, const TCOD_ColorRGB* c2) {
+  const int dr = (int)c1->r - c2->r;
+  const int dg = (int)c1->g - c2->g;
+  const int db = (int)c1->b - c2->b;
   return dr * dr + dg * dg + db * db;
 }
 
-void getPattern(const TCOD_color_t desired[4], TCOD_color_t palette[2], int* __restrict nbCols, int* __restrict ascii) {
-  /* adapted from Jeff Lait's code posted on r.g.r.d */
-  int flag = 0;
+// Return a new tile graphic from 4 quadrant colors.
+static TCOD_ConsoleTile generate_quadrant_graphic(const TCOD_ColorRGB desired[4]) {
+  // adapted from Jeff Lait's code posted on r.g.r.d
   /*
-    pixels have following flag values :
+    A quadrant bitmask.  A raised bit means the quadrant is part of the foreground.
+    Quadrants are arranged with the following bits:
       X 1
       2 4
-    flag indicates which pixels uses foreground color (palette[1])
-  */
-  static int flagToAscii[8] = {
+   */
+  int quadrant_mask = 0;
+  // Maps a mask of quadrants to a codepoint.  A negative codepoint means to swap the fg/bg values.
+  static int quadrant_to_codepoint[8] = {
       0,
-      0x259D,   // Quadrant upper right.
-      0x2597,   // Quadrant lower left.
+      0x259D,  // Quadrant upper right.
+      0x2597,  // Quadrant lower left.
       -0x259A,  // Quadrant upper left and lower right.
-      0x2596,   // Quadrant lower right.
-      0x2590,   // Right half block.
+      0x2596,  // Quadrant lower right.
+      0x2590,  // Right half block.
       -0x2580,  // Upper half block.
-      -0x2598   // Quadrant upper left.
+      -0x2598  // Quadrant upper left.
   };
-  int weight[2] = {0, 0};
-  int i;
-
-  /* First colour trivial. */
-  palette[0] = desired[0];
-
-  /* Ignore all duplicates... */
-  for (i = 1; i < 4; i++) {
-    if (!TCOD_color_equals(desired[i], palette[0])) {
-      break;
+  int quadrant_index;  // The active color quadrant being checked.
+  // Ignore all quadrants matching the first color.
+  for (quadrant_index = 1; quadrant_index < 4; ++quadrant_index) {
+    if (!TCOD_color_equals(desired[quadrant_index], desired[0])) {
+      break;  // Found a second color, will continue to check for colors from this index.
     }
   }
-
-  /* All the same. */
-  if (i == 4) {
-    *nbCols = 1;
-    return;
+  if (quadrant_index == 4) {  // This tile is a solid color.
+    return (TCOD_ConsoleTile){
+        .ch = ' ',
+        .fg = {desired[0].r, desired[0].g, desired[0].b, 255},
+        .bg = {desired[0].r, desired[0].g, desired[0].b, 255},
+    };
   }
-  weight[0] = i;
-
-  /* Found a second colour... */
-  palette[1] = desired[i];
-  weight[1] = 1;
-  flag |= 1 << (i - 1);
-  *nbCols = 2;
+  TCOD_ColorRGB palette[2] = {desired[0], desired[quadrant_index]};  // The current color palette: {bg color, fg color}.
+  int weight[2] = {quadrant_index, 1};  // Number of quadrants that each pallette color is assigned, respectively.
+  quadrant_mask |= 1 << (quadrant_index - 1);
   /* remaining colours */
-  ++i;
-  while (i < 4) {
-    if (TCOD_color_equals(desired[i], palette[0])) {
+  ++quadrant_index;
+  while (quadrant_index < 4) {
+    if (TCOD_color_equals(desired[quadrant_index], palette[0])) {
+      // Assign to the background color.
       ++weight[0];
-    } else if (TCOD_color_equals(desired[i], palette[1])) {
-      flag |= 1 << (i - 1);
+    } else if (TCOD_color_equals(desired[quadrant_index], palette[1])) {
+      // Assign to the foreground color.
+      quadrant_mask |= 1 << (quadrant_index - 1);
       ++weight[1];
     } else {
-      /* Bah, too many colours, */
-      /* merge the two nearest */
-      int dist0i = rgbdist(&desired[i], &palette[0]);
-      int dist1i = rgbdist(&desired[i], &palette[1]);
-      int dist01 = rgbdist(&palette[0], &palette[1]);
-      if (dist0i < dist1i) {
-        if (dist0i <= dist01) {
-          /* merge 0 and i */
-          palette[0] = TCOD_color_lerp(desired[i], palette[0], weight[0] / (1.0f + weight[0]));
+      // No more than two colors can be supported, so merge colors based on the smallest differences.
+      const int dist_0_q = rgb_squared_distance(&desired[quadrant_index], &palette[0]);
+      const int dist_1_q = rgb_squared_distance(&desired[quadrant_index], &palette[1]);
+      const int dist_0_1 = rgb_squared_distance(&palette[0], &palette[1]);
+      if (dist_0_q < dist_1_q) {
+        if (dist_0_q <= dist_0_1) {
+          // Merge 0 and quadrant_index.
+          palette[0] = TCOD_color_lerp(desired[quadrant_index], palette[0], weight[0] / (1.0f + weight[0]));
           ++weight[0];
         } else {
-          /* merge 0 and 1 */
+          // Merge 0 and 1.
           palette[0] = TCOD_color_lerp(palette[0], palette[1], (float)weight[1] / (weight[0] + weight[1]));
           ++weight[0];
-          palette[1] = desired[i];
-          flag = 1 << (i - 1);
+          palette[1] = desired[quadrant_index];
+          quadrant_mask = 1 << (quadrant_index - 1);
         }
       } else {
-        if (dist1i <= dist01) {
-          /* merge 1 and i */
-          palette[1] = TCOD_color_lerp(desired[i], palette[1], weight[1] / (1.0f + weight[1]));
+        if (dist_1_q <= dist_0_1) {
+          // Merge 1 and quadrant_index.
+          palette[1] = TCOD_color_lerp(desired[quadrant_index], palette[1], weight[1] / (1.0f + weight[1]));
           ++weight[1];
-          flag |= 1 << (i - 1);
+          quadrant_mask |= 1 << (quadrant_index - 1);
         } else {
-          /* merge 0 and 1 */
+          // Merge 0 and 1.
           palette[0] = TCOD_color_lerp(palette[0], palette[1], (float)weight[1] / (weight[0] + weight[1]));
           ++weight[0];
-          palette[1] = desired[i];
-          flag = 1 << (i - 1);
+          palette[1] = desired[quadrant_index];
+          quadrant_mask = 1 << (quadrant_index - 1);
         }
       }
     }
-    ++i;
+    ++quadrant_index;
+  }
+  if (quadrant_to_codepoint[quadrant_mask] >= 0) {
+    return (TCOD_ConsoleTile){
+        .ch = quadrant_to_codepoint[quadrant_mask],
+        .fg = (TCOD_ColorRGBA){palette[1].r, palette[1].g, palette[1].b, 255},
+        .bg = (TCOD_ColorRGBA){palette[0].r, palette[0].g, palette[0].b, 255},
+    };
+  } else {  // A negative codepoint means we invert the bg/fg colors.
+    return (TCOD_ConsoleTile){
+        .ch = -quadrant_to_codepoint[quadrant_mask],
+        .fg = (TCOD_ColorRGBA){palette[0].r, palette[0].g, palette[0].b, 255},
+        .bg = (TCOD_ColorRGBA){palette[1].r, palette[1].g, palette[1].b, 255},
+    };
   }
-  *ascii = flagToAscii[flag];
 }
 
 void TCOD_image_blit_2x(
-    const TCOD_Image* __restrict image, TCOD_Console* __restrict con, int dx, int dy, int sx, int sy, int w, int h) {
-  if (!image) {
-    return;
-  }
-  con = TCOD_console_validate_(con);
-  if (!con) {
-    return;
-  }
-  TCOD_color_t grid[4];
-  TCOD_color_t cols[2];
+    const TCOD_Image* __restrict image,
+    TCOD_Console* __restrict console,
+    int dest_x,
+    int dest_y,
+    int src_x,
+    int src_y,
+    int src_width,
+    int src_height) {
+  if (!image) return;
+  console = TCOD_console_validate_(console);
+  if (!console) return;
 
-  int width, height;
-  TCOD_image_get_size(image, &width, &height);
-  if (w == -1) {
-    w = width;
-  }
-  if (h == -1) {
-    h = height;
-  }
+  int img_width, img_height;
+  TCOD_image_get_size(image, &img_width, &img_height);
+  if (src_width == -1) src_width = img_width;
+  if (src_height == -1) src_height = img_height;
 
-  /* check that the sx,sy/w,h rectangle is inside the image */
-  TCOD_ASSERT(sx >= 0 && sy >= 0 && sx + w <= width && sy + h <= height);
-  TCOD_IFNOT(w > 0 && h > 0) { return; }
+  /* check that the src_x,src_y/src_width,src_height rectangle is inside the image */
+  TCOD_ASSERT(src_x >= 0 && src_y >= 0 && src_x + src_width <= img_width && src_y + src_height <= img_height);
+  TCOD_IFNOT(src_width > 0 && src_height > 0) { return; }
 
-  sx = MAX(0, sx);
-  sy = MAX(0, sy);
-  w = MIN(w, width - sx);
-  h = MIN(h, height - sy);
+  src_x = MAX(0, src_x);
+  src_y = MAX(0, src_y);
+  src_width = MIN(src_width, img_width - src_x);
+  src_height = MIN(src_height, img_height - src_y);
 
-  int max_x = dx + w / 2 <= con->w ? w : (con->w - dx) * 2;
-  int max_y = dy + h / 2 <= con->h ? h : (con->h - dy) * 2;
+  int max_x = dest_x + src_width / 2 <= console->w ? src_width : (console->w - dest_x) * 2;
+  int max_y = dest_y + src_height / 2 <= console->h ? src_height : (console->h - dest_y) * 2;
   /* check that the image is not blitted outside the console */
-  TCOD_IFNOT(dx + max_x / 2 >= 0 && dy + max_y / 2 >= 0 && dx < con->w && dy < con->h) { return; }
-  max_x += sx;
-  max_y += sy;
+  TCOD_IFNOT(dest_x + max_x / 2 >= 0 && dest_y + max_y / 2 >= 0 && dest_x < console->w && dest_y < console->h) {
+    return;
+  }
+  max_x += src_x;
+  max_y += src_y;
 
-  for (int cx = sx; cx < max_x; cx += 2) {
-    for (int cy = sy; cy < max_y; cy += 2) {
+  for (int img_x = src_x; img_x < max_x; img_x += 2) {
+    for (int img_y = src_y; img_y < max_y; img_y += 2) {
+      TCOD_ColorRGB grid[4];
       /* get the 2x2 super pixel colors from the image */
-      int con_x = dx + (cx - sx) / 2;
-      int con_y = dy + (cy - sy) / 2;
-      TCOD_color_t consoleBack = TCOD_console_get_char_background(con, con_x, con_y);
-      grid[0] = TCOD_image_get_pixel(image, cx, cy);
+      const int console_x = dest_x + (img_x - src_x) / 2;
+      const int console_y = dest_y + (img_y - src_y) / 2;
+      TCOD_ColorRGB consoleBack = TCOD_console_get_char_background(console, console_x, console_y);
+      grid[0] = TCOD_image_get_pixel(image, img_x, img_y);
       if (image->has_key_color && TCOD_color_equals(grid[0], image->key_color)) {
         grid[0] = consoleBack;
       }
-      if (cx < max_x - 1) {
-        grid[1] = TCOD_image_get_pixel(image, cx + 1, cy);
+      if (img_x < max_x - 1) {
+        grid[1] = TCOD_image_get_pixel(image, img_x + 1, img_y);
         if (image->has_key_color && TCOD_color_equals(grid[1], image->key_color)) {
           grid[1] = consoleBack;
         }
       } else {
         grid[1] = consoleBack;
       }
-      if (cy < max_y - 1) {
-        grid[2] = TCOD_image_get_pixel(image, cx, cy + 1);
+      if (img_y < max_y - 1) {
+        grid[2] = TCOD_image_get_pixel(image, img_x, img_y + 1);
         if (image->has_key_color && TCOD_color_equals(grid[2], image->key_color)) {
           grid[2] = consoleBack;
         }
       } else {
         grid[2] = consoleBack;
       }
-      if (cx < max_x - 1 && cy < max_y - 1) {
-        grid[3] = TCOD_image_get_pixel(image, cx + 1, cy + 1);
+      if (img_x < max_x - 1 && img_y < max_y - 1) {
+        grid[3] = TCOD_image_get_pixel(image, img_x + 1, img_y + 1);
         if (image->has_key_color && TCOD_color_equals(grid[3], image->key_color)) {
           grid[3] = consoleBack;
         }
       } else {
         grid[3] = consoleBack;
       }
-      /* analyse color, posterize, get pattern */
-      int nbCols;
-      int ascii;
-      getPattern(grid, cols, &nbCols, &ascii);
-      if (nbCols == 1) {
-        /* single color */
-        TCOD_console_set_char_background(con, con_x, con_y, cols[0], TCOD_BKGND_SET);
-        TCOD_console_set_char(con, con_x, con_y, ' ');
-      } else if (ascii >= 0) {
-        TCOD_console_set_char_background(con, con_x, con_y, cols[0], TCOD_BKGND_SET);
-        TCOD_console_set_char_foreground(con, con_x, con_y, cols[1]);
-        TCOD_console_set_char(con, con_x, con_y, ascii);
-      } else {
-        /* negative ascii code means we need to invert back/fore colors */
-        TCOD_console_set_char_background(con, con_x, con_y, cols[1], TCOD_BKGND_SET);
-        TCOD_console_set_char_foreground(con, con_x, con_y, cols[0]);
-        TCOD_console_set_char(con, con_x, con_y, -ascii);
-      }
+      /* analyze color, posterize, get pattern */
+      console->tiles[console_y * console->w + console_x] = generate_quadrant_graphic(grid);
     }
   }
 }
