@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,12 +30,11 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 // clang-format off
-/*
- * This is a libtcod internal module.
- * Use at your own risks...
- */
-#ifndef _TCOD_LEX_HPP
-#define _TCOD_LEX_HPP
+/// This is a libtcod internal module.
+/// Use at your own risk.
+/// @cond INTERNAL
+#ifndef TCOD_LEX_HPP_
+#define TCOD_LEX_HPP_
 
 #include "lex.h"
 
@@ -63,8 +62,8 @@ class TCODLIB_API TCODLex {
   bool expect(int tokenType);
   bool expect(int tokenType, const char* tokenValue);
 
-  void savepoint(TCODLex* savept);
-  void restore(TCODLex* savept);
+  void savepoint(TCODLex* savepoint);
+  void restore(TCODLex* savepoint);
   char* getLastJavadoc();
 
   int getFileLine()
@@ -116,4 +115,5 @@ class TCODLIB_API TCODLex {
  protected:
   TCOD_lex_t* data;
 };
-#endif // _TCOD_LEX_HPP
+#endif // TCOD_LEX_HPP_
+/// @endcond
