@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,8 +29,8 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-#ifndef _LIBTCOD_H
-#define _LIBTCOD_H
+#ifndef LIBTCOD_H_
+#define LIBTCOD_H_
 
 #include "bresenham.h"
 #include "bsp.h"
@@ -50,6 +50,7 @@
 #include "image.h"
 #include "lex.h"
 #include "list.h"
+#include "logging.h"
 #include "mersenne.h"
 #include "mouse.h"
 #include "namegen.h"
@@ -60,9 +61,6 @@
 #include "pathfinder_frontier.h"
 #include "portability.h"
 #include "random.h"
-#include "renderer_gl.h"
-#include "renderer_gl1.h"
-#include "renderer_gl2.h"
 #include "renderer_sdl2.h"
 #include "sdl2/event.h"
 #include "sys.h"
@@ -82,6 +80,10 @@
 #include "bsp.hpp"
 #include "color.hpp"
 #include "console.hpp"
+#include "console_printing.hpp"
+#include "console_rexpaint.hpp"
+#include "console_types.hpp"
+#include "context.hpp"
 #include "fov.hpp"
 #include "heightmap.hpp"
 #include "image.hpp"
@@ -94,9 +96,12 @@
 #include "parser.hpp"
 #include "path.hpp"
 #include "sys.hpp"
+#include "tileset.hpp"
+#include "tileset_bdf.hpp"
+#include "tileset_fallback.hpp"
 #include "tree.hpp"
 #include "txtfield.hpp"
 #include "zip.hpp"
 #endif  // __cplusplus
 
-#endif
+#endif  // LIBTCOD_H_
