@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,10 +31,14 @@
  */
 #include "mouse.hpp"
 
+#ifndef NO_SDL
 void TCODMouse::showCursor(bool visible) { TCOD_mouse_show_cursor(visible); }
-
 bool TCODMouse::isCursorVisible() { return TCOD_mouse_is_cursor_visible() != 0; }
-
 void TCODMouse::move(int x, int y) { TCOD_mouse_move(x, y); }
-
 TCOD_mouse_t TCODMouse::getStatus() { return TCOD_mouse_get_status(); }
+#else
+void TCODMouse::showCursor(bool visible) {}
+bool TCODMouse::isCursorVisible() { return false; }
+void TCODMouse::move(int x, int y) {}
+TCOD_mouse_t TCODMouse::getStatus() { return {}; }
+#endif  // NO_SDL
