@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,8 @@
 #ifndef _TCOD_PERLIN_HPP
 #define _TCOD_PERLIN_HPP
 
+#include <utility>
+
 #include "mersenne.hpp"
 #include "noise.h"
 #include "noise_defaults.h"
@@ -125,6 +127,14 @@ class TCODLIB_API TCODNoise {
 		TCODNoise(int dimensions, float hurst, float lacunarity, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
 		TCODNoise(int dimensions, float hurst, float lacunarity, TCODRandom *random, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
 
+    TCODNoise(const TCODNoise&) = delete;
+    TCODNoise& operator=(const TCODNoise&) = delete;
+    TCODNoise(TCODNoise&& rhs) noexcept { std::swap(data, rhs.data); };
+    TCODNoise& operator=(TCODNoise&& rhs) noexcept {
+      std::swap(data, rhs.data);
+      return *this;
+    };
+
 		/**
 		@PageName noise_init
 		@FuncDesc To release resources used by a generator, use those functions :
@@ -142,14 +152,14 @@ class TCODLIB_API TCODNoise {
 			delete noise;
 		@CEx
 			// create a generator
-			TCOD_noise_t noise = TCOD_noise_new(2,TCOD_NOISE_DEFAULT_HURST, TCOD_NOISE_DEFAUT_LACUNARITY, NULL);
+			TCOD_noise_t noise = TCOD_noise_new(2,TCOD_NOISE_DEFAULT_HURST, TCOD_NOISE_DEFAULT_LACUNARITY, NULL);
 			// use it
 			...
 			// destroy it
 			TCOD_noise_delete(noise);
 		@PyEx
 			# create a generator
-			noise = libtcod.noise_new(2,libtcod.NOISE_DEFAULT_HURST, libtcod.NOISE_DEFAUT_LACUNARITY, 0)
+			noise = libtcod.noise_new(2,libtcod.NOISE_DEFAULT_HURST, libtcod.NOISE_DEFAULT_LACUNARITY, 0)
 			# use it
 			...
 			# destroy it
@@ -227,6 +237,7 @@ float TCOD_noise_get_ex(TCOD_noise_t noise, float *f, TCOD_noise_type_t type)
 			value = libtcod.noise_get(noise2d,[0.5,0.7], libtcod.NOISE_PERLIN)
    		*/
 		float get(float *f, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
+		float get(const float *f, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
 		/**
 		@PageName noise_get_fbm
 		@PageFather noise
@@ -274,6 +285,7 @@ float TCOD_noise_get_fbm(TCOD_noise_t noise, float *f, float octaves, TCOD_noise
 			value = libtcod.noise_get_fbm(noise2d,[0.5,0.7],32.0, libtcod.NOISE_PERLIN)
 		*/
 		float getFbm(float *f, float octaves, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
+		float getFbm(const float *f, float octaves, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
 		/**
 		@PageName noise_get_turbulence
 		@PageFather noise
@@ -320,6 +332,7 @@ float TCOD_noise_get_turbulence_ex(TCOD_noise_t noise, float *f, float octaves,
 			value = libtcod.noise_get_turbulence(noise2d,[0.5,0.7],32.0,libtcod.NOISE_PERLIN)
 		*/
 		float getTurbulence(float *f, float octaves, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
+		float getTurbulence(const float *f, float octaves, TCOD_noise_type_t type = TCOD_NOISE_DEFAULT);
 
 	protected :
 		friend class TCODLIB_API TCODHeightMap;
