@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -86,7 +86,7 @@ bool new_struct(TCOD_parser_struct_t def, const char* name) noexcept {
       return listener->parserNewStruct(parser, i_def.get(), name) ? 1 : 0;
     }
   }
-  // not found. autodeclaring struct
+  // not found. auto-declaring struct
   parser->defs.push_back(std::make_unique<TCODParserStruct>());
   parser->defs.back()->data = def;
   return listener->parserNewStruct(parser, parser->defs.back().get(), name) ? 1 : 0;
@@ -130,7 +130,7 @@ void TCODParser::error(const char* msg, ...) {
   char buf[2048];
   va_list ap;
   va_start(ap, msg);
-  vsprintf(buf, msg, ap);
+  vsnprintf(buf, sizeof(buf), msg, ap);
   va_end(ap);
   TCOD_parser_error("%s", buf);
 }
