@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -165,7 +165,7 @@ public :
 	@Param name	The name of the structure type (in the example, this would be "item_type").
 	@CppEx
 		TCODParser parser();
-		TCODParserStruct *itemTypeStruct = parser.newStructrue("item_type");
+		TCODParserStruct *itemTypeStruct = parser.newStructure("item_type");
 	@CEx
 		TCOD_parser_t parser = TCOD_parser_new();
 		TCOD_parser_struct_t item_type_struct = TCOD_parser_new_struct(parser, "item_type");
