@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,14 +35,14 @@
 #include <cstdio>
 
 #include "console_init.h"
-#include "error.h"
+#include "error.hpp"
 #include "libtcod_int.h"
 
+#ifndef NO_SDL
 void TCODSystem::sleepMilli(uint32_t milliseconds) { TCOD_sys_sleep_milli(milliseconds); }
-
 uint32_t TCODSystem::getElapsedMilli() { return TCOD_sys_elapsed_milli(); }
-
 float TCODSystem::getElapsedSeconds() { return TCOD_sys_elapsed_seconds(); }
+float TCODSystem::getLastFrameLength() { return TCOD_sys_get_last_frame_length(); }
 
 void TCODSystem::saveScreenshot(const char* filename) { TCOD_sys_save_screenshot(filename); }
 
@@ -72,8 +72,6 @@ void TCODSystem::setFps(int val) { TCOD_sys_set_fps(val); }
 
 int TCODSystem::getFps() { return TCOD_sys_get_fps(); }
 
-float TCODSystem::getLastFrameLength() { return TCOD_sys_get_last_frame_length(); }
-
 void TCODSystem::getCurrentResolution(int* w, int* h) { TCOD_sys_get_current_resolution(w, h); }
 
 void TCODSystem::getFullscreenOffsets(int* offset_x, int* offset_y) {
@@ -102,10 +100,10 @@ TCOD_list_t TCODSystem::getDirectoryContent(const char* path, const char* patter
 bool TCODSystem::fileExists(const char* filename, ...) {
   FILE* in;
   bool ret = false;
-  char f[1024];
+  char f[1024] = "";
   va_list ap;
   va_start(ap, filename);
-  vsprintf(f, filename, ap);
+  vsnprintf(f, sizeof(f), filename, ap);
   va_end(ap);
   in = fopen(f, "rb");
   if (in != NULL) {
@@ -128,6 +126,7 @@ bool TCODSystem::setClipboard(const char* value) { return TCOD_sys_clipboard_set
 
 char* TCODSystem::getClipboard() { return TCOD_sys_clipboard_get(); }
 
+#ifndef TCOD_NO_THREADS
 // thread stuff
 int TCODSystem::getNumCores() { return TCOD_sys_get_num_cores(); }
 
@@ -165,6 +164,7 @@ void TCODSystem::broadcastCondition(TCOD_cond_t cond) { TCOD_condition_broadcast
 void TCODSystem::waitCondition(TCOD_cond_t cond, TCOD_mutex_t mut) { TCOD_condition_wait(cond, mut); }
 
 void TCODSystem::deleteCondition(TCOD_cond_t cond) { TCOD_condition_delete(cond); }
+#endif  // TCOD_NO_THREADS
 
 // custom post-renderer
 static ITCODSDLRenderer* post_renderer = NULL;
@@ -177,3 +177,9 @@ void TCODSystem::registerSDLRenderer(ITCODSDLRenderer* renderer) {
   ::post_renderer = renderer;
   TCOD_sys_register_SDL_renderer(TCOD_CRenderer);
 }
+#else
+void TCODSystem::sleepMilli(uint32_t) {}
+uint32_t TCODSystem::getElapsedMilli() { return 0; }
+float TCODSystem::getElapsedSeconds() { return 0; }
+float TCODSystem::getLastFrameLength() { return 0; }
+#endif  // NO_SDL
