@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -87,6 +87,7 @@ TCOD_DEPRECATED_NOMESSAGE
 TCODLIB_API bool TCOD_sys_read_file(const char* filename, unsigned char** buf, size_t* size);
 TCOD_DEPRECATED_NOMESSAGE
 TCODLIB_API bool TCOD_sys_write_file(const char* filename, unsigned char* buf, uint32_t size);
+#ifndef TCOD_NO_THREADS
 /* threads */
 TCOD_DEPRECATED_NOMESSAGE
 TCODLIB_API TCOD_thread_t TCOD_thread_new(int (*func)(void*), void* data);
@@ -125,6 +126,8 @@ TCOD_DEPRECATED_NOMESSAGE
 TCODLIB_API void TCOD_condition_wait(TCOD_cond_t sem, TCOD_mutex_t mut);
 TCOD_DEPRECATED_NOMESSAGE
 TCODLIB_API void TCOD_condition_delete(TCOD_cond_t sem);
+#endif  // TCOD_NO_THREADS
+#ifndef NO_SDL
 /* dynamic library */
 typedef void* TCOD_library_t;
 TCOD_DEPRECATED_NOMESSAGE
@@ -134,24 +137,86 @@ TCODLIB_API void* TCOD_get_function_address(TCOD_library_t library, const char*
 TCOD_DEPRECATED_NOMESSAGE
 TCODLIB_API void TCOD_close_library(TCOD_library_t);
 
-#ifndef NO_SDL
+/***************************************************************************
+    @brief Alias for SDL_GetTicks.
+
+    \rst
+    .. deprecated:: 1.19
+        You should call SDL_GetTicks directly.
+    \endrst
+ */
+TCOD_DEPRECATED("Use SDL_GetTicks instead.")
 TCODLIB_API uint32_t TCOD_sys_elapsed_milli(void);
+/***************************************************************************
+    @brief Returns the number of seconds since the start of the program.
+
+    \rst
+    .. deprecated:: 1.19
+        Use SDL_GetTicks and convert the result into seconds instead of using this function.
+    \endrst
+ */
+TCOD_DEPRECATED("Use SDL_GetTicks instead.")
 TCODLIB_API float TCOD_sys_elapsed_seconds(void);
+/***************************************************************************
+    @brief Alias for SDL_Delay.
+
+    \rst
+    .. deprecated:: 1.19
+        You should call SDL_Delay directly.
+    \endrst
+ */
+TCOD_DEPRECATED("Use SDL_Delay instead.")
 TCODLIB_API void TCOD_sys_sleep_milli(uint32_t val);
+/***************************************************************************
+    @brief Set the desired framerate.
+
+    \rst
+    .. deprecated:: 1.19
+        This function will not affect libtcod contexts.
+        Set the framerate with :any:`tcod::Timer` instead.
+    \endrst
+ */
+TCOD_DEPRECATED("This function is not compatible with contexts.  Use tcod::Timer or SDL timing functions instead.")
 TCODLIB_API void TCOD_sys_set_fps(int val);
+/***************************************************************************
+    @brief Get the current framerate.
+
+    \rst
+    .. deprecated:: 1.19
+        This function will not work with libtcod contexts.
+        Use :any:`tcod::Timer` instead.
+    \endrst
+ */
+TCOD_DEPRECATED("This function is not compatible with contexts.  Use tcod::Timer or SDL timing functions instead.")
 TCODLIB_API int TCOD_sys_get_fps(void);
+/***************************************************************************
+    @brief Get the delta time between the last two frames.
+
+    \rst
+    .. deprecated:: 1.19
+        This function will not work with libtcod contexts.
+        Use :any:`tcod::Timer` instead.
+    \endrst
+ */
+TCOD_DEPRECATED("This function is not compatible with contexts.  Use tcod::Timer or SDL timing functions instead.")
 TCODLIB_API float TCOD_sys_get_last_frame_length(void);
 
+TCOD_DEPRECATED("This function is not compatible with contexts.")
 TCODLIB_API void TCOD_sys_save_screenshot(const char* filename);
+TCOD_DEPRECATED("This function is not compatible with contexts.")
 TCODLIB_API void TCOD_sys_force_fullscreen_resolution(int width, int height);
+TCOD_DEPRECATED("This function is not compatible with contexts.")
 TCODLIB_API TCOD_NODISCARD int TCOD_sys_set_renderer(TCOD_renderer_t renderer);
+TCOD_DEPRECATED("This function is not compatible with contexts.")
 TCODLIB_API TCOD_renderer_t TCOD_sys_get_renderer(void);
 /**
     Return the resolution of the current monitor.
  */
 TCOD_DEPRECATED("Use SDL to determine the screen resolution instead.")
 TCODLIB_API TCOD_Error TCOD_sys_get_current_resolution(int* w, int* h);
+TCOD_DEPRECATED("This function is not compatible with contexts.")
 TCODLIB_API void TCOD_sys_get_fullscreen_offsets(int* offset_x, int* offset_y);
+TCOD_DEPRECATED("This function is not compatible with contexts.")
 TCODLIB_API void TCOD_sys_get_char_size(int* w, int* h);
 
 /**
