@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,30 +43,13 @@
 #include "error.h"
 #include "globals.h"
 #include "libtcod_int.h"
+#include "logging.h"
 #include "mouse.h"
 #include "parser.h"
 #include "sys.h"
 #include "tileset.h"
 
 #ifndef NO_SDL
-#ifdef __ANDROID__
-#define TCOD_TOUCH_INPUT
-#define MAX_TOUCH_FINGERS 5
-
-typedef struct {
-  int nupdates;                            /* how many updates have happened since the first finger was pressed. */
-  uint32_t ticks0;                         /* current number of ticks at start of touch event sequence. */
-  SDL_FingerID finger_id;                  /* the last finger which was pressed. */
-  int coords[MAX_TOUCH_FINGERS][2];        /* absolute position of each finger. */
-  int coords_delta[MAX_TOUCH_FINGERS][2];  /* absolute position of each finger. */
-  int consolecoords[MAX_TOUCH_FINGERS][2]; /* cell coordinates in the root console for each finger. */
-  int consolecoords_delta[MAX_TOUCH_FINGERS][2]; /* cell coordinates in the root console for each finger. */
-  int nfingers;                                  /* number of unique fingers employed at any time during this. */
-  int nfingerspressed;                           /* number of those fingers currently still pressed. */
-  SDL_FingerID finger_ids[MAX_TOUCH_FINGERS];
-  char fingerspressed[MAX_TOUCH_FINGERS];
-} TCOD_touch_t;
-#endif  // __ANDROID__
 
 /* to enable bitmap locking. Is there any use ?? makes the OSX port renderer to fail */
 /*#define USE_SDL_LOCKS */
@@ -75,9 +58,11 @@ typedef struct {
 bool TCOD_sys_check_bmp(const char* filename);
 SDL_Surface* TCOD_sys_read_bmp(const char* filename);
 TCOD_Error TCOD_sys_write_bmp(SDL_Surface* surf, const char* filename);
+#ifndef TCOD_NO_PNG
 bool TCOD_sys_check_png(const char* filename);
 SDL_Surface* TCOD_sys_read_png(const char* filename);
 TCOD_Error TCOD_sys_write_png(SDL_Surface* surf, const char* filename);
+#endif  // TCOD_NO_PNG
 
 typedef struct {
   const char* extension;
@@ -88,7 +73,9 @@ typedef struct {
 
 static image_support_t image_type[] = {
     {"BMP", TCOD_sys_check_bmp, TCOD_sys_read_bmp, TCOD_sys_write_bmp},
+#ifndef TCOD_NO_PNG
     {"PNG", TCOD_sys_check_png, TCOD_sys_read_png, TCOD_sys_write_png},
+#endif  // TCOD_NO_PNG
     {NULL, NULL, NULL, NULL},
 };
 
@@ -194,7 +181,7 @@ TCOD_Error TCOD_sys_load_player_config(void) {
     /* custom font */
     if (TCOD_sys_file_exists(font)) {
       int fontNbCharHoriz, fontNbCharVertic;
-      strcpy(TCOD_ctx.font_file, font);
+      strncpy(TCOD_ctx.font_file, font, sizeof(TCOD_ctx.font_file) - 1);
       TCOD_ctx.font_in_row = TCOD_parser_get_bool_property(parser, "libtcod.fontInRow");
       TCOD_ctx.font_greyscale = TCOD_parser_get_bool_property(parser, "libtcod.fontGreyscale");
       TCOD_ctx.font_tcod_layout = TCOD_parser_get_bool_property(parser, "libtcod.fontTcodLayout");
@@ -220,7 +207,7 @@ TCOD_Error TCOD_sys_load_player_config(void) {
       printf("Warning : font file '%s' does not exist\n", font);
     }
   }
-  printf("Warning: The use of libtcod.cfg is deprecated.\n");
+  TCOD_log_warning("The use of libtcod.cfg is deprecated.");
   TCOD_set_error("The use of libtcod.cfg is deprecated.");
   return TCOD_E_WARN;
 }
@@ -276,7 +263,7 @@ void TCOD_sys_save_screenshot(const char* filename) {
   while (!filename) {
     /* generate filename */
     FILE* access_file = NULL;
-    sprintf(buf, "./screenshot%03d.png", idx);
+    snprintf(buf, sizeof(buf), "./screenshot%03d.png", idx);
     access_file = fopen(buf, "rb");
     if (!access_file) {
       filename = buf;
@@ -596,7 +583,7 @@ static TCOD_key_t TCOD_sys_SDLtoTCOD(const SDL_Event* ev, int flags) {
   ret->pressed = 0;
   switch (ev->type) {
     case SDL_KEYUP: {
-      TCOD_key_t tmpkey;
+      TCOD_key_t tmp_key;
       switch (ev->key.keysym.sym) {
         case SDLK_LALT:
           ret->lalt = 0;
@@ -625,16 +612,16 @@ static TCOD_key_t TCOD_sys_SDLtoTCOD(const SDL_Event* ev, int flags) {
         default:
           break;
       }
-      TCOD_sys_convert_event(ev, &tmpkey);
+      TCOD_sys_convert_event(ev, &tmp_key);
       if (flags & TCOD_KEY_RELEASED) {
-        ret->vk = tmpkey.vk;
-        ret->c = tmpkey.c;
+        ret->vk = tmp_key.vk;
+        ret->c = tmp_key.c;
         ret->pressed = 0;
       }
       break;
     }
     case SDL_KEYDOWN: {
-      TCOD_key_t tmpkey;
+      TCOD_key_t tmp_key;
       switch (ev->key.keysym.sym) {
         case SDLK_LALT:
           ret->lalt = 1;
@@ -663,10 +650,10 @@ static TCOD_key_t TCOD_sys_SDLtoTCOD(const SDL_Event* ev, int flags) {
         default:
           break;
       }
-      TCOD_sys_convert_event(ev, &tmpkey);
+      TCOD_sys_convert_event(ev, &tmp_key);
       if (flags & TCOD_KEY_PRESSED) {
-        ret->vk = tmpkey.vk;
-        ret->c = tmpkey.c;
+        ret->vk = tmp_key.vk;
+        ret->c = tmp_key.c;
         ret->pressed = 1;
       }
       break;
@@ -814,8 +801,8 @@ void TCOD_sys_unproject_screen_coords(int sx, int sy, int* ssx, int* ssy) {
         (scale_data.src_y0 +
          ((sy - scale_data.dst_offset_y) * scale_data.src_copy_width) / scale_data.dst_display_width);
   } else {
-    *ssx = sx - TCOD_ctx.fullscreen_offsetx;
-    *ssy = sy - TCOD_ctx.fullscreen_offsety;
+    *ssx = sx - TCOD_ctx.fullscreen_offset_x;
+    *ssy = sy - TCOD_ctx.fullscreen_offset_y;
   }
 }
 
@@ -844,8 +831,8 @@ void TCOD_sys_pixel_to_tile(double* x, double* y) {
   if (TCOD_ctx.engine && TCOD_ctx.engine->c_pixel_to_tile_) {
     TCOD_ctx.engine->c_pixel_to_tile_(TCOD_ctx.engine, x, y);
   } else {
-    *x = (*x - TCOD_ctx.fullscreen_offsetx) / TCOD_ctx.font_width;
-    *y = (*y - TCOD_ctx.fullscreen_offsety) / TCOD_ctx.font_height;
+    *x = (*x - TCOD_ctx.fullscreen_offset_x) / TCOD_ctx.font_width;
+    *y = (*y - TCOD_ctx.fullscreen_offset_y) / TCOD_ctx.font_height;
   }
 }
 /**
@@ -1108,7 +1095,6 @@ TCOD_event_t TCOD_sys_wait_for_event(int eventMask, TCOD_key_t* key, TCOD_mouse_
  *  \param eventMask A bit-mask of TCOD_event_t flags.
  *  \param key Optional pointer to a TCOD_key_t struct.
  *  \param mouse Optional pointer to a TCOD_mouse_t struct.
- *  \param flush This should always be false.
  *  \return A TCOD_event_t flag showing which event was actually processed.
  */
 TCOD_event_t TCOD_sys_check_for_event(int eventMask, TCOD_key_t* key, TCOD_mouse_t* mouse) {
@@ -1256,7 +1242,7 @@ TCOD_Error TCOD_sys_get_current_resolution(int* w, int* h) {
       return TCOD_set_errorvf("SDL error: %s", SDL_GetError());
     }
   }
-  // Temporarally load the video subsystem if it isn't already active.
+  // Temporarily load the video subsystem if it isn't already active.
   if (SDL_InitSubSystem(SDL_INIT_VIDEO) < 0) {
     return TCOD_set_errorvf("SDL error: %s", SDL_GetError());
   }
@@ -1356,7 +1342,7 @@ bool TCOD_sys_file_exists(const char* filename, ...) {
   char f[1024];
   va_list ap;
   va_start(ap, filename);
-  vsprintf(f, filename, ap);
+  vsnprintf(f, sizeof(f), filename, ap);
   va_end(ap);
   SDL_RWops* rwops;
   rwops = SDL_RWFromFile(f, "rb");
