@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,30 +29,42 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-// clang-format off
 #ifndef _TCOD_TREE_HPP
 #define _TCOD_TREE_HPP
 
 #include "tree.h"
 
 class TCODLIB_API TCODTree {
-public :
-	TCODTree *next;
-	TCODTree *father;
-	TCODTree *sons;
+ public:
+  TCODTree* next{};
+  TCODTree* father{};
+  TCODTree* sons{};
 
-	TCODTree() : next(NULL),father(NULL),sons(NULL){}
-	void addSon(TCODTree *data) {
-		data->father=this;
-		TCODTree *lastson = sons;
-		while ( lastson && lastson->next ) lastson=lastson->next;
-		if ( lastson ) {
-			lastson->next=data;
-		} else {
-			sons=data;
-		}
-	}
+  TCODTree() = default;
+  TCODTree(const TCODTree&) = delete;
+  TCODTree& operator=(const TCODTree&) = delete;
+  TCODTree(TCODTree&& rhs) noexcept { (*this) = std::move(rhs); };
+  TCODTree& operator=(TCODTree&& rhs) noexcept {
+    swap(*this, rhs);
+    return *this;
+  };
 
+  void addSon(TCODTree* data) {
+    data->father = this;
+    TCODTree* last_son = sons;
+    while (last_son && last_son->next) last_son = last_son->next;
+    if (last_son) {
+      last_son->next = data;
+    } else {
+      sons = data;
+    }
+  }
+
+  friend void swap(TCODTree& lhs, TCODTree& rhs) noexcept {
+    std::swap(lhs.next, rhs.next);
+    std::swap(lhs.father, rhs.father);
+    std::swap(lhs.sons, rhs.sons);
+  };
 };
 
 #endif
