@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,10 +33,10 @@
 #define LIBTCOD_VERSION_H
 
 #define TCOD_MAJOR_VERSION  1
-#define TCOD_MINOR_VERSION  18
-#define TCOD_PATCHLEVEL     1
+#define TCOD_MINOR_VERSION  24
+#define TCOD_PATCHLEVEL     0
 
-#define TCOD_STRVERSION "1.18.1"
+#define TCOD_STRVERSION "1.24.0"
 
 /**
     Converts version numbers into a numeric value.
@@ -76,5 +76,13 @@
     \endrst
  */
 #define TCOD_STRVERSIONNAME "libtcod " TCOD_STRVERSION
+/***************************************************************************
+    @brief Returns true if the compiled version of libtcod is at least (major, minor).
+
+    \rst
+    .. versionadded:: 1.19
+    \endrst
+ */
+#define TCOD_VERSION_ATLEAST(major, minor) (TCOD_COMPILEDVERSION >= TCOD_VERSIONNUM(major, minor, 0))
 
 #endif /* LIBTCOD_VERSION_H */
