@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include "zip.hpp"
+#ifndef TCOD_NO_ZLIB
 
 TCODZip::TCODZip() { data = TCOD_zip_new(); }
 
@@ -75,7 +76,7 @@ int TCODZip::getData(int nbBytes, void* pdata) { return TCOD_zip_get_data(data,
 
 TCODColor TCODZip::getColor() { return TCODColor(TCOD_zip_get_color(data)); }
 
-TCODImage* TCODZip::getImage() { return new TCODImage(TCOD_zip_get_image(data)); }
+TCODImage* TCODZip::getImage() { return new TCODImage{tcod::ImagePtr{TCOD_zip_get_image(data)}}; }
 
 TCODConsole* TCODZip::getConsole() { return new TCODConsole(TCOD_zip_get_console(data)); }
 
@@ -86,3 +87,4 @@ uint32_t TCODZip::getCurrentBytes() const { return TCOD_zip_get_current_bytes(da
 uint32_t TCODZip::getRemainingBytes() const { return TCOD_zip_get_remaining_bytes(data); }
 
 void TCODZip::skipBytes(uint32_t nbBytes) { TCOD_zip_skip_bytes(data, nbBytes); }
+#endif  // TCOD_NO_ZLIB
