@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2021, Jice and the libtcod contributors.
+ * Copyright © 2008-2023, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@
  */
 #ifndef _TCOD_ZIP_H
 #define _TCOD_ZIP_H
+#ifndef TCOD_NO_ZLIB
 
 #include "color.h"
 #include "console_types.h"
@@ -54,8 +55,8 @@ TCODLIB_API void TCOD_zip_put_int(TCOD_zip_t zip, int val);
 TCODLIB_API void TCOD_zip_put_float(TCOD_zip_t zip, float val);
 TCODLIB_API void TCOD_zip_put_string(TCOD_zip_t zip, const char* val);
 TCODLIB_API void TCOD_zip_put_color(TCOD_zip_t zip, const TCOD_color_t val);
-TCODLIB_API void TCOD_zip_put_image(TCOD_zip_t zip, const TCOD_image_t val);
-TCOD_DEPRECATED("This function will fail with console characters greater than 255.")
+TCODLIB_API void TCOD_zip_put_image(TCOD_zip_t zip, const TCOD_Image* val);
+TCOD_DEPRECATED("This function will corrupt console characters greater than 255.")
 TCODLIB_API void TCOD_zip_put_console(TCOD_zip_t zip, const TCOD_Console* val);
 /**
     Write a TCOD_Random* object.
@@ -75,7 +76,7 @@ TCODLIB_API int TCOD_zip_get_int(TCOD_zip_t zip);
 TCODLIB_API float TCOD_zip_get_float(TCOD_zip_t zip);
 TCODLIB_API const char* TCOD_zip_get_string(TCOD_zip_t zip);
 TCODLIB_API TCOD_color_t TCOD_zip_get_color(TCOD_zip_t zip);
-TCODLIB_API TCOD_image_t TCOD_zip_get_image(TCOD_zip_t zip);
+TCODLIB_API TCOD_Image* TCOD_zip_get_image(TCOD_zip_t zip);
 TCODLIB_API TCOD_console_t TCOD_zip_get_console(TCOD_zip_t zip);
 /**
     Read a TCOD_Random* object.
@@ -83,11 +84,12 @@ TCODLIB_API TCOD_console_t TCOD_zip_get_console(TCOD_zip_t zip);
     .. versionadded:: 1.16
     \endrst
  */
-TCODLIB_API TCOD_random_t TCOD_zip_get_random(TCOD_zip_t zip);
+TCODLIB_API TCOD_Random* TCOD_zip_get_random(TCOD_zip_t zip);
 TCODLIB_API int TCOD_zip_get_data(TCOD_zip_t zip, int nbBytes, void* data);
 TCODLIB_API uint32_t TCOD_zip_get_remaining_bytes(TCOD_zip_t zip);
 TCODLIB_API void TCOD_zip_skip_bytes(TCOD_zip_t zip, uint32_t nbBytes);
 #ifdef __cplusplus
 }
 #endif
-#endif
+#endif  // TCOD_NO_ZLIB
+#endif  // _TCOD_ZIP_H
